"""Tests for SQLAlchemy support"""

import cherrypy, os, sys, threading, turbogears

from sqlalchemy import MetaData, Table, Column, ForeignKey, Integer, String
try:
    from sqlalchemy.ext.activemapper import ActiveMapper, column, one_to_many
    if sys.version_info < (2, 4):
        try:
            class TestActiveMapper(ActiveMapper):
                class mapping:
                    main_id = column(Integer)
                    test_id = column(Integer,
                        foreign_key=ForeignKey('testactivemapper.main_id'))
        except AttributeError:
            # known problem with ActiveMapper 0.3.10 and Py 2.3,
            # pretend ActiveMapper is not installed in this case
            raise ImportError
except ImportError: # SQLAlchemy >= 0.5
    ActiveMapper = None
    from sqlalchemy.orm import relation

from turbogears import config, redirect, expose, errorhandling
from turbogears.database import get_engine, metadata, session, mapper
from turbogears.controllers import RootController
from turbogears.testutil import create_request, sqlalchemy_cleanup, \
    capture_log, print_log


# Fixture

class User(object):
    def __repr__(self):
        return "(User %s, password %s)" % (self.user_name, self.password)


def setup_module():
    global fresh_metadata, users_table, test_table, Person, Address, Test

    config.update({
        "sqlalchemy.dburi" : "sqlite:///:memory:"})

    if os.path.exists('freshtest.db'):
        os.unlink('freshtest.db')

    get_engine()
    fresh_metadata = MetaData()
    # :memory: can't be used in multiple threads
    fresh_metadata.bind = 'sqlite:///freshtest.db'
    metadata.bind.echo = True
    fresh_metadata.bind.echo = True

    users_table = Table("users", metadata,
        Column("user_id", Integer, primary_key=True),
        Column("user_name", String(40)),
        Column("password", String(10)))

    mapper(User, users_table)

    if ActiveMapper:

        class Person(ActiveMapper):
            class mapping:
                id = column(Integer, primary_key=True)
                name = column(String(40))
                addresses = one_to_many("Address")

        class Address(ActiveMapper):
            class mapping:
                id = column(Integer, primary_key=True)
                address = column(String(40))
                city = column(String(40))
                person_id = column(Integer,
                    foreign_key=ForeignKey("person.id"))

    else:

        persons_table = Table("persons", metadata,
                Column("id", Integer, primary_key=True),
                Column("name", String(40)))
        addresses_table = Table("addresses", metadata,
                Column("id", Integer, primary_key=True),
                Column("address", String(40)),
                Column("city", String(40)),
                Column("person_id", Integer,
                    ForeignKey(persons_table.c.id)))

        class Person(object):
            pass

        class Address(object):
            pass

        mapper(Person, persons_table)
        mapper(Address, addresses_table, properties=dict(
            person=relation(Person, backref='addresses')))

    test_table = Table("test", fresh_metadata,
        Column("id", Integer, primary_key=True),
        Column("val", String(40)))

    class Test(object):
        pass

    mapper(Test, test_table)

    try:
        metadata.create_all()
        fresh_metadata.create_all()
    except Exception, error:
        # workaround for a problem with PySqlite < 2.6 and SQLAlchemy < 0.5
        if 'Cannot operate on a closed cursor' in str(error):
            metadata.create_all(checkfirst=False)
            fresh_metadata.create_all(checkfirst=False)
        else:
            raise

def teardown_module():
    metadata.drop_all()
    fresh_metadata.drop_all()
    fresh_metadata.bind.dispose()
    if os.path.exists('freshtest.db'):
        os.unlink('freshtest.db')
    sqlalchemy_cleanup()


# Simple database tests

def test_query_in_session():
    i = users_table.insert()
    i.execute(user_name="globbo", password="thegreat!")
    query = session.query(User)
    globbo = query.filter_by(user_name="globbo").one()
    assert globbo.password == "thegreat!"
    users_table.delete().execute()

def test_create_and_query():
    i = users_table.insert()
    i.execute(user_name="globbo", password="thegreat!")
    s = users_table.select()
    r = s.execute()
    assert len(r.fetchall()) == 1
    users_table.delete().execute()

def test_active_mapper():
    p = Person(name="Ford Prefect")
    a = Address(address="1 West Guildford", city="Betelgeuse")
    p.addresses.append(a)
    session.flush()
    try:
        session.expunge_all()
    except AttributeError: # SQLAlchemy < 0.5.1
        session.clear()
    q = session.query(Person)
    ford = q.filter_by(name="Ford Prefect").one()
    assert ford is not p
    assert len(ford.addresses) == 1


# Exception handling

class MyRoot(RootController):
    """A small root controller for our exception handling tests"""

    [expose()]
    def no_error(self, name):
        """Test controller"""
        Person(name=name)
        raise redirect("/confirm")

    def e_handler(self, tg_exceptions=None):
        """Test error handler"""
        cherrypy.response.code = 501
        return "An exception occurred: %r (%s)" % ((tg_exceptions,)*2)

    [expose()]
    def create_person(self, id, docom=0, doerr=0, doflush=0):
        """Test controller"""
        Person(id=id)
        if int(docom):
            cherrypy.request.sa_transaction.commit()
        if int(doerr) == 1:
            raise Exception('User generated exception')
        if int(doerr) == 2:
            raise turbogears.redirect('/')
        if int(doflush):
            try:
                session.flush()
            except Exception:
                if int(doflush) == 1:
                    raise
        return "No exceptions occurred"
    create_person = errorhandling.exception_handler(e_handler)(create_person)


def test_implicit_trans_no_error():
    """If a controller runs sucessfully, the transaction is commited."""
    capture_log("turbogears.database")
    cherrypy.root = MyRoot()
    create_request("/no_error?name=A.%20Dent")
    print_log()
    try:
        session.expunge_all()
    except AttributeError: # SQLAlchemy < 0.5.1
        session.clear()
    q = session.query(Person)
    arthur = q.filter_by(name="A. Dent").one()

def test_raise_sa_exception():
    """If a controller causes an SA exception, it is raised properly."""
    capture_log("turbogears.database")
    cherrypy.root = MyRoot()
    create_request("/create_person?id=20")
    output = cherrypy.response.body[0]
    assert 'No exceptions occurred' in output
    create_request("/create_person?id=20")
    output = cherrypy.response.body[0]
    # SA 0.3 uses SQLError, 0.4 DBAPIError, 0.5 IntegrityError
    assert ('SQLError' in output or 'DBAPIError' in output
        or 'IntegrityError' in output)

def test_user_exception():
    """If a controller raises an exception, transactions are rolled back."""
    cherrypy.root = MyRoot()
    create_request("/create_person?id=21&doerr=1")
    try: # should be done automatically, but just in case
        session.expunge_all()
    except AttributeError: # SQLAlchemy < 0.5.1
        session.clear()
    query = Person.query
    if ActiveMapper:
        query = query()
    assert query.get(21) is None

def test_user_redirect():
    """If a controller redirects, transactions are committed."""
    cherrypy.root = MyRoot()
    create_request("/create_person?id=22&doerr=2")
    try: # should be done automatically, but just in case
        session.expunge_all()
    except AttributeError: # SQLAlchemy < 0.5.1
        session.clear()
    query = Person.query
    if ActiveMapper:
        query = query()
    assert query.get(22) is not None

def test_cntrl_commit():
    """It's safe to commit a transaction in the controller."""
    cherrypy.root = MyRoot()
    create_request("/create_person?id=23&docom=1")
    assert 'InvalidRequestError' not in cherrypy.response.body[0]

def test_cntrl_flush():
    """It's safe to flush in the controller."""
    cherrypy.root = MyRoot()
    create_request("/create_person?id=24&doflush=1")
    assert 'No exceptions occurred' in cherrypy.response.body[0]
    create_request("/create_person?id=24&doflush=0")
    assert 'IntegrityError' in cherrypy.response.body[0]
    create_request("/create_person?id=24&doflush=1")
    assert 'IntegrityError' in cherrypy.response.body[0]
    create_request("/create_person?id=24&doflush=2")
    assert 'No exceptions occurred' in cherrypy.response.body[0]


# Exception handling with rollback

class RbRoot(RootController):
    """A small root controller for our transaction rollback tests"""

    def handerr(self, id):
        """Test error handler"""
        Person(id=int(id)+1)
        return dict()

    [expose()]
    def doerr(self, id, dorb=0):
        """Test controller"""
        Person(id=id)
        if int(dorb):
            cherrypy.request.sa_transaction.rollback()
        raise Exception('test')
    doerr = errorhandling.exception_handler(handerr)(doerr)


def test_exc_rollback():
    """"An exception within a controller method causes a rollback."""
    cherrypy.root = RbRoot()
    create_request('/doerr?id=25')
    query = Person.query
    if ActiveMapper:
        query = query()
    assert query.get(25) is None
    assert query.get(26) is not None

def test_exc_done_rollback():
    """No problems with error handler if controller manually rollbacks."""
    cherrypy.root = RbRoot()
    create_request('/doerr?id=27&dorb=1')
    assert cherrypy.response.body[0] == '{"tg_flash": null}'


# Session freshness tests

class FreshRoot(RootController):
    """A small root controller for our session freshness tests"""

    [expose()]
    def test1(self):
        assert session.query(Test).get(1).val == 'a'
        return dict()

    [expose()]
    def test2(self):
        session.query(Test).get(1).val = 'b'
        return dict()

    [expose()]
    def test3(self):
        assert session.query(Test).get(1).val == 'b'
        return dict()


def test_session_freshness():
    """Check for session freshness.

    Changes made to the data in thread B should be reflected in thread A.

    """
    fresh_metadata.bind.execute(test_table.insert(), dict(id=1, val='a'))
    cherrypy.root = FreshRoot()
    create_request("/test1")
    assert cherrypy.response.status.startswith("200")
    assert 'AssertionError' not in cherrypy.response.body[0]
    # Call test2 in a different thread
    class ThreadB(threading.Thread):
        def run(self):
            create_request("/test2")
            assert cherrypy.response.status.startswith("200")
            assert 'AssertionError' not in cherrypy.response.body[0]
    thrdb = ThreadB()
    thrdb.start()
    thrdb.join()
    create_request("/test3")
    assert cherrypy.response.status.startswith("200")
    assert 'AssertionError' not in cherrypy.response.body[0]



"""Commands for listing TurboGears default and extension packages info"""

try:
    from email.parser import Parser
except ImportError: # Python < 2.5
    from email.Parser import Parser

import pkg_resources

entrypoints = {"tg-admin Commands" : "turbogears.command",
    "Template Engines" : "python.templating.engines",
    "Widget Packages" : "turbogears.widgets",
    "TurboGears Extensions" : "turbogears.extensions",
    "Identity Providers" : "turbogears.identity.provider",
    "Visit Managers" : "turbogears.visit.manager",
    "Toolbox Gadgets" : "turbogears.toolboxcommand"}

parsestr = Parser().parsestr

def retrieve_pkg_info(distribution):
    """Retrieve parsed package info from distribution."""
    return parsestr(distribution.get_metadata('PKG-INFO'))


def retrieve_url(distribution):
    """Retrieve URL from distribution."""
    try:
        info = retrieve_pkg_info(distribution)
    except Exception:
        url = None
    else:
        url = info['Home-page'] or info['Url']  or info['Download-Url']
    return url


def add_link(distribution):
    """Add link to distribution."""
    info = str(distribution)
    url = retrieve_url(distribution)
    if url:
        info = str(info).split(None, 1)
        info[0] = '<a href="%s">%s</a>' % (url, info[0])
        info = ' '.join(info)
    return info


def retrieve_info(with_links=False):
    """Retrieve default and extension packages info."""
    format = with_links and add_link or str
    # get default packages
    packages = [format(pkg) for pkg in pkg_resources.require("Turbogears")]
    # get extension packages
    plugins = {}
    for name, pointname in entrypoints.items():
        plugins[name] = ["%s (%s)" % (entrypoint.name, format(entrypoint.dist))
            for entrypoint in pkg_resources.iter_entry_points(pointname)]
    return packages, plugins


class InfoCommand:
    """Shows version info for debugging."""

    desc = "Show version info"

    def __init__(self,*args, **kwargs):
        pass

    def run(self):
        print """TurboGears Complete Version Information

TurboGears requires:
"""
        packages, plugins = retrieve_info()
        for p in packages:
            print '*', p
        for name, pluginlist in plugins.items():
            print "\n", name, "\n"
            for plugin in pluginlist:
                print '*', plugin




from turbogears.i18n.utils import *

def test_get_accept_languages():
    assert get_accept_languages("da, en-gb;q=0.8, en;q=0.7") == ["da", "en_GB", "en"]
    assert get_accept_languages("da;q=0.6, en-gb;q=1.0, en;q=0.7") == ["en_GB", "en", "da"]




import copy
import threading
import warnings

from inspect import isclass
from itertools import count, ifilter
from new import instancemethod

import kid

from turbogears import validators
from turbogears.util import setlike
from formencode.schema import Schema

try:
    set
except NameError: # Python 2.3
    from sets import Set as set

__all__ = ["MetaWidget", "load_kid_template"]

param_prefix = '_param_'

class MetaWidget(type):
    """The meta class for widgets."""

    def __new__(cls, name, bases, dct):
        # update_data has been deprecated
        if 'update_data' in dct and name != "Widget":
            warnings.warn(
                "update_data has been renamed update_params, please "
                "rename your method. "
                "Note: this warning will be removed once 1.0 is "
                "released and your actual code will stop working.",
                DeprecationWarning, 2)
        # Makes sure we get the union of params and member_widgets
        # from all our bases.
        params = setlike(dct.get('params', []))
        # template_vars has been deprecated
        if 'template_vars' in dct:
            params.add_all(dct['template_vars'])
            warnings.warn(
                "Use of template_vars inside a widget is deprecated, "
                "use params instead. "
                "Note: this warning will be removed once 1.0 is "
                "released and your actual code will stop working.",
                DeprecationWarning, 2)
        member_widgets = setlike(dct.get('member_widgets', []))
        compound = False
        for base in bases:
            params.add_all(getattr(base, 'params', []))
            if getattr(base, 'compound', False):
                member_widgets.add_all(getattr(base, 'member_widgets', []))
                compound = True
        for param in params:
            # Swap all params listed at 'params' with a ParamDescriptor
            try:
                dct[param_prefix+param] = dct[param]
                dct[param] = ParamDescriptor(param)
            except KeyError:
                # declared in a superclass, skip it...
                pass
        params = list(params)
        dct['params'] = params
        #XXX: Remove when deprecation is effective
        dct['template_vars'] = params
        if compound:
            dct['member_widgets'] = list(member_widgets)
        # Pick params_doc from all bases giving priority to the widget's own
        params_doc = {}
        for base in bases:
            params_doc.update(getattr(base, 'params_doc', {}))
        params_doc.update(dct.get('params_doc', {}))
        dct['params_doc'] = params_doc
        return super(MetaWidget, cls).__new__(cls, name, bases, dct)

    def __init__(cls, name, bases, dct):
        if "__init__" in dct:
            dct["__init__"] = _decorate_widget_init(dct["__init__"])
            cls.__init__ = dct["__init__"]
        super(MetaWidget, cls).__init__(name, bases, dct)
        modname = "%s.%s" % (cls.__module__, name)
        if cls.template:
            (cls.template_c,
             cls.template) = load_kid_template(cls.template, modname)
        cls._locked = False

#############################################################################
# Method decorators and other MetaWidget helpers                            #
#############################################################################

class ParamDescriptor(object):
    """Descriptor to support automatic callable support for widget params."""
    def __init__(self, param_name):
        self.param_name = param_prefix + param_name

    def __get__(self, obj, typ=None):
        if obj is None:
            # return the original class attribute. This makes the descriptor
            # almost invisible
            return getattr(typ, self.param_name)
        param = getattr(obj, self.param_name)
        if callable(param):
            return param()
        return param

    def __set__(self, obj, value):
        setattr(obj, self.param_name, value)

def lockwidget(self, *args, **kw):
    """Set this widget as locked the first time it's displayed."""
    gotlock = self._displaylock.acquire(False)
    if gotlock:
        del self.display
        self._locked = True
    output = self.__class__.display(self, *args, **kw)
    if gotlock:
        self._displaylock.release()
    return output

def _decorate_widget_init(func):
    """Decorator for a widgets __init__ method.

    Ensures that the display method for the instance is overridden by
    lockwidget, that an eventual validator dict is applied to the widget
    validator and that a validation schema is generated for compound widgets.

    """
    def widget_init(self, *args, **kw):
        self.display = instancemethod(lockwidget, self, self.__class__)
        # We may want to move the InputWidget related logic to another
        # decorator
        input_widget = hasattr(self, "validator")
        # Makes sure we only generate a schema once in the all __init__
        # cooperative calls from subclasses. Same for the displaylock
        if not hasattr(self, '__initstack'):
            self._displaylock = threading.Lock()
            self.__initstack = []
        else:
            self.__initstack.append(True)
        # Manage an eventual validator dictionary that provides additional
        # parameters to the default validator (if present).
        # We remove it from kw and apply it after the execution of the
        # decorated __init__ method since only at this point we can check
        # for the presence of a default validator
        if input_widget and ("validator" in kw) \
           and isinstance(kw["validator"], dict):
            validator_dict = kw.pop("validator")
        else:
            validator_dict = None
        # Execute the decorated __init__ method
        func(self, *args, **kw)
        try:
            self.__initstack.pop()
        except IndexError:
            # We're the first __init__ called
            del self.__initstack
            if input_widget:
                # Apply an eventual validator dictionary
                if validator_dict:
                    if self.validator is not None:
                        class_validator = self.__class__.validator
                        if self.validator is class_validator:
                            # avoid modifying a validator that has been
                            # defined at class level and therefor is
                            # shared by all instances of this widget
                            self.validator = copy.deepcopy(class_validator)
                        self.validator.__dict__.update(validator_dict)
                    else:
                        raise ValueError, ("You can't use a dictionary to "
                                           "provide additional parameters "
                                           "as the widget doesn't provide a "
                                           "default validator" )
                # Generate the validation Schema if we are a compound widget
                if getattr(self, 'compound', False):
                    widgets = self.iter_member_widgets()
                    # generate the schema associated to our widgets
                    validator = generate_schema(self.validator, widgets)
                    if getattr(self, 'repeating', False):
                        self.validator = validators.ForEach(validator)
                    else:
                        self.validator = validator
    return widget_init

#############################################################################
# Widget template support.                                                  #
#############################################################################

# Keeps a count of the widget instances which have overrided their template on
# the constructor so we can generate unique template-module names
widget_instance_serial = count()

# XXX: Maybe Widget should have a __del__ method to unload any template that
#      has been loaded during the instances initialization?
def load_kid_template(t, modname=None):
    """Load the given template into the given module name.

    If modname is None, an unique one will be generated.
    Returns a tuple (compiled_tmpl, template_text (or modulepath).

    """
    if isinstance(t, basestring) and "<" in t:
        if not modname:
            modname = 'instance_template_%d' % widget_instance_serial.next()
        return (kid.load_template(t, name=modname).Template, t)
    else:
        return (t, None)

#############################################################################
# Schema generation support                                                 #
#############################################################################
class NullValidator(validators.FancyValidator):
    """A do-nothing validator.

    Used as a placeholder for fields with no validator so they don't get
    stripped by the Schema.

    """
    if_missing = None

def copy_schema(schema):
    """Recursively copy a schema."""
    new_schema = copy.copy(schema)
    new_schema.pre_validators = schema.pre_validators[:]
    new_schema.chained_validators = schema.chained_validators[:]
    fields = {}
    for k, v in schema.fields.iteritems():
        if isinstance(v, Schema):
            v = copy_schema(v)
        fields[k] = v
    new_schema.fields = fields
    return new_schema

def merge_schemas(to_schema, from_schema, inplace=False):
    """Recursively merge from_schema into to_schema

    Takes care of leaving to_schema intact if inplace is False (default).
    Returns a new Schema instance if inplace is False or to_schema is a Schema
    class not an instance or the changed to_schema.

    """
    # Nested schemas may be Schema (sub-)classes instead of instances.
    # Copying Schema classes does not work, so we just create an instance.
    if isclass(to_schema) and issubclass(to_schema, validators.Schema):
        to_schema = to_schema()
    elif not inplace:
        to_schema = copy_schema(to_schema)

    # Recursively merge child schemas
    is_schema = lambda f: isinstance(f[1], validators.Schema)
    seen = set()
    for k, v in ifilter(is_schema, to_schema.fields.iteritems()):
        seen.add(k)
        from_field = from_schema.fields.get(k)
        if from_field:
            v = merge_schemas(v, from_field)
            to_schema.add_field(k, v)

    # Add remaining fields if we can
    can_add = lambda f: f[0] not in seen and can_add_field(to_schema, f[0])
    for field in ifilter(can_add, from_schema.fields.iteritems()):
        to_schema.add_field(*field)

    return to_schema

def add_field_to_schema(schema, widget):
    """Add widget's validator if any to the given schema."""
    name = widget.name
    if widget.validator is not None:
        if isinstance(widget.validator, validators.Schema):
            # Schema instance or class, might need to merge 'em...
            if widget.name in schema.fields:
                assert (isinstance(schema.fields[name], validators.Schema) or
                    issubclass(schema.fields[name], validators.Schema)
                    ), "Validator for '%s' should be a Schema subclass" % name
                v = merge_schemas(schema.fields[name], widget.validator)
            else:
                v = widget.validator
            schema.add_field(name, v)
        elif can_add_field(schema, name):
            # Non-schema validator, add it if we can...
            schema.add_field(name, widget.validator)
    elif can_add_field(schema, name):
        schema.add_field(name, NullValidator())

def generate_schema(schema, widgets):
    """Generate or extend a copy of schema with all validators of all widgets.

    schema may be a schema instance or class, widgets is a list of widgets
    instances.
    Returns the new schema instance.

    """
    if schema is None:
        schema = validators.Schema()
    elif isclass(schema) and issubclass(schema, validators.Schema):
        # allows to attach Schema (sub-)classes to widgets.
        schema = schema()
    else:
        schema = copy_schema(schema)
    for widget in widgets:
        if widget.is_named:
            add_field_to_schema(schema, widget)
    return schema

def can_add_field(schema, field_name):
    """Checks if we can safely add a field.

    Makes sure we're not overriding any field in the Schema. NullValidators
    are ok to override.

    """
    current_field = schema.fields.get(field_name)
    return bool(current_field is None or
                isinstance(current_field, NullValidator))



import sys
import os
import codecs
import pygettext

MESSAGES = []

def detect_unicode_encoding(bytes):
    encodings_map = [
        (3, codecs.BOM_UTF8, 'UTF-8'),
        (4, codecs.BOM_UTF32_LE, 'UTF-32LE'),
        (4, codecs.BOM_UTF32_BE, 'UTF-32BE'),
        (2, codecs.BOM_UTF16_LE, 'UTF-16LE'),
        (2, codecs.BOM_UTF16_BE, 'UTF-16BE'),
    ]
    for (offset, bom, name) in encodings_map:
        if bytes[:offset] == bom:
            return name, offset

    return 'UTF-8', 0

class ParseError(ValueError):
    """Signals an error reading .po file."""

def merge(master_file, language_files):
    parsed_master_file = parse(master_file)
    for path in language_files:
        merging(parsed_master_file, path)

def merging(parsed_master_file, path):
    lang_file = parse(path)
    id_map = {}
    new_lang = []
    for msg in lang_file:
        id_map[msg['id']] = msg['message']

    for msg in parsed_master_file:
        msg['message'] = id_map.get(msg['id'])
        new_lang.append(msg)

    save(path, new_lang)

def items(path, sort_by, dir):
    po = parse(path)
    po = po[1:]
    if sort_by:
        return sort(po, sort_by, dir)

    return po

def sort(po, sort_by, dir):
    group = dict()
    sorted = list()
    col_map  = dict(id='id', string='message', context='path')
    for message in po:
        group.setdefault(message[col_map[sort_by]], []).append(message)

    kg = group.keys()
    kg.sort()
    if dir == 'up':
        kg.reverse()

    for k in kg:
        sorted.extend(group[k])

    return sorted

def save(path, message_list):
    txt = []
    m = message_list[0]['message']
    txt.append(m)
    txt.append(u'\n\n')

    for p in message_list[1:]:
        message = p['message'] or ''
        context = p['context']
        id = p['id']
        txt.append(u'#: %s' % context)
        txt.append(u'msgid %s\n' % normalize(id))
        txt.append(u'msgstr %s\n\n' % normalize(message))

    txt = u''.join(txt)
    backup_name = path.replace('.po', '.back')

    try:
        os.remove(backup_name)

    except os.error:
        pass

    os.rename(path, backup_name)
    codecs.open(path, 'wb', 'utf-8').write(txt)

def update(path, msg_id, msg_text):
    message_list = parse(path)
    for p in message_list[1:]:
        if p['id'].strip() == msg_id.strip():
            p['message'] = msg_text
    save(path, message_list)

def quote(msg):
    return pygettext.escape_unicode(msg)

def normalize(s):
    # taken from pygettext module but changed a bit
    lines = s.split('\n')
    if len(lines) == 1:
        s = '"' + quote(s) + '"'

    else:
        if not lines[-1]:
            del lines[-1]
            lines[-1] = lines[-1] + '\n'

        for i in range(len(lines)):
            lines[i] = quote(lines[i])

        lineterm = '\\n"\n"'
        s = '""\n"' + lineterm.join(lines) + '"'

    return s

def add(id, str, context, fuzzy, MESSAGES):
    "Add a non-fuzzy translation to the dictionary."
    if fuzzy:
        return

    c = context.split(':')
    path = c[0]
    file = os.path.basename(path)
    line = c[-1].replace('\n','') #remove the \n

    MESSAGES.append(dict(id=id,
                    message=str,
                    path=path,
                    context=context,
                    file=file,
                    line=line
                    ))

def parse(infile):
    MESSAGES = list()
    ID = 1
    STR = 2
    header = list()

    fd = open(infile, 'rt')
    encoding, offset = detect_unicode_encoding(fd.read(4))
    fd.seek(offset)
    lines = [line.decode(encoding) for line in fd.readlines()]

    section = None
    fuzzy = 0

    # Parse the catalog
    lno = 0
    context = ''
    prev_context = ''
    heading = True
    for l in lines:
        if not l:
            continue

        lno += 1
        if heading:
            if l.startswith('#: '):
                heading = False

            if l.startswith('msgid "') and header and \
                    'Generated-By:' in header[-1]:
                heading = False

            if l.strip() and heading:
                header.append(l)

        # If we get a comment line after a msgstr, this is a new entry
        if l[0] == '#' and section == STR:
            add(msgid, msgstr, prev_context, fuzzy, MESSAGES)
            section = None
            fuzzy = 0

        # Record a fuzzy mark
        if l[:2] == '#,' and l.find('fuzzy'):
            fuzzy = 1

        if l.startswith('#: '):
            context = l[len('#: '):]

        # Skip comments
        if l[0] == '#':
            continue

        # Now we are in a msgid section, output previous section
        if l.startswith('msgid'):
            if section == STR:
                add(msgid, msgstr, prev_context, fuzzy, MESSAGES)

            section = ID
            prev_context = context
            l = l[5:]
            msgid = msgstr = ''

        # Now we are in a msgstr section
        elif l.startswith('msgstr'):
            section = STR
            l = l[6:]

        # Skip empty lines
        l = l.strip()
        if not l:
            continue

        # XXX: Does this always follow Python escape semantics?
        try:
            l = eval(l)
        except Exception, e:
            print >> sys.stderr, 'Escape error on %s: %d' % (infile, lno), \
                'before:', repr(l)
            raise ParseError(e)

        try:
            l = l.decode('utf8')
        except UnicodeDecodeError:
            print >> sys.stderr, 'Encoding error on %s: %d' % (infile, lno), \
                'before:', repr(l)
            raise ParseError(e)

        if section == ID:
            msgid += l

        elif section == STR:
            msgstr += l

        else:
            print >> sys.stderr, 'Syntax error on %s:%d' % (infile, lno), \
                  'before:'
            print >> sys.stderr, l
            raise ParseError(e)

    # Add last entry
    if section == STR:
        add(msgid, msgstr, prev_context, fuzzy, MESSAGES)

    MESSAGES[0]['message'] = u''.join(header)
    return MESSAGES



from turbogears.widgets.base import *
from turbogears.widgets.rpc import *
from turbogears.widgets.forms import *
from turbogears.widgets.big_widgets import *
from turbogears.widgets.datagrid import *
from turbogears.widgets.i18n import *
from turbogears.widgets.links import *



"""
TurboGears internationalization/localization module.
"""

from turbogears.i18n.tg_gettext import gettext, ngettext, install, \
    is_locale_supported, lazy_gettext, lazy_ngettext, plain_gettext
from turbogears.i18n.utils import get_locale, get_accept_languages, \
    set_session_locale, google_translate
from turbogears.i18n.format import get_countries, get_country, \
    get_month_names, get_abbr_month_names, get_weekday_names, \
    get_abbr_weekday_names, get_languages, format_date, format_number, \
    format_decimal, format_currency, parse_number, parse_decimal
from turbogears.i18n.kidutils import translate, i18n_filter



#! /usr/bin/env python
# -*- coding: iso-8859-1 -*-
# Originally written by Barry Warsaw <barry@zope.com>
#
# Minimally patched to make it even more xgettext compatible
# by Peter Funk <pf@artcom-gmbh.de>
#
# 2002-11-22 Jürgen Hermann <jh@web.de>
# Added checks that _() only contains string literals, and
# command line args are resolved to module lists, i.e. you
# can now pass a filename, a module or package name, or a
# directory (including globbing chars, important for Win32).
# Made docstring fit in 80 chars wide displays using pydoc.
#

# for selftesting
import re
try:
    import fintl
    _ = fintl.gettext
except ImportError:
    _ = lambda s: s

__doc__ = _("""pygettext -- Python equivalent of xgettext(1)

Many systems (Solaris, Linux, Gnu) provide extensive tools that ease the
internationalization of C programs. Most of these tools are independent of
the programming language and can be used from within Python programs.
Martin von Loewis' work[1] helps considerably in this regard.

There's one problem though; xgettext is the program that scans source code
looking for message strings, but it groks only C (or C++). Python
introduces a few wrinkles, such as dual quoting characters, triple quoted
strings, and raw strings. xgettext understands none of this.

Enter pygettext, which uses Python's standard tokenize module to scan
Python source code, generating .pot files identical to what GNU xgettext[2]
generates for C and C++ code. From there, the standard GNU tools can be
used.

A word about marking Python strings as candidates for translation. GNU
xgettext recognizes the following keywords: gettext, dgettext, dcgettext,
and gettext_noop. But those can be a lot of text to include all over your
code. C and C++ have a trick: they use the C preprocessor. Most
internationalized C source includes a #define for gettext() to _() so that
what has to be written in the source is much less. Thus these are both
translatable strings:

    gettext("Translatable String")
    _("Translatable String")

Python of course has no preprocessor so this doesn't work so well.  Thus,
pygettext searches only for _() by default, but see the -k/--keyword flag
below for how to augment this.

 [1] http://www.python.org/workshops/1997-10/proceedings/loewis.html
 [2] http://www.gnu.org/software/gettext/gettext.html

NOTE: pygettext attempts to be option and feature compatible with GNU
xgettext where ever possible. However some options are still missing or are
not fully implemented. Also, xgettext's use of command line switches with
option arguments is broken, and in these cases, pygettext just defines
additional switches.

Usage: pygettext [options] inputfile ...

Options:

    -a
    --extract-all
        Extract all strings.

    -d name
    --default-domain=name
        Rename the default output file from messages.pot to name.pot.

    -E
    --escape
        Replace non-ASCII characters with octal escape sequences.

    -D
    --docstrings
        Extract module, class, method, and function docstrings.  These do
        not need to be wrapped in _() markers, and in fact cannot be for
        Python to consider them docstrings. (See also the -X option).

    -h
    --help
        Print this help message and exit.

    -k word
    --keyword=word
        Keywords to look for in addition to the default set, which are:
        %(DEFAULTKEYWORDS)s

        You can have multiple -k flags on the command line.

    -K
    --no-default-keywords
        Disable the default set of keywords (see above).  Any keywords
        explicitly added with the -k/--keyword option are still recognized.

    --no-location
        Do not write filename/lineno location comments.

    -n
    --add-location
        Write filename/lineno location comments indicating where each
        extracted string is found in the source.  These lines appear before
        each msgid.  The style of comments is controlled by the -S/--style
        option.  This is the default.

    -o filename
    --output=filename
        Rename the default output file from messages.pot to filename.  If
        filename is `-' then the output is sent to standard out.

    -p dir
    --output-dir=dir
        Output files will be placed in directory dir.

    -S stylename
    --style stylename
        Specify which style to use for location comments.  Two styles are
        supported:

        Solaris  # File: filename, line: line-number
        GNU      #: filename:line

        The style name is case insensitive.  GNU style is the default.

    -v
    --verbose
        Print the names of the files being processed.

    -V
    --version
        Print the version of pygettext and exit.

    -w columns
    --width=columns
        Set width of output to columns.

    -x filename
    --exclude-file=filename
        Specify a file that contains a list of strings that are not be
        extracted from the input files.  Each string to be excluded must
        appear on a line by itself in the file.

    -X filename
    --no-docstrings=filename
        Specify a file that contains a list of files (one per line) that
        should not have their docstrings extracted.  This is only useful in
        conjunction with the -D option above.

If `inputfile' is -, standard input is read.
""")

import os
import imp
import sys
import glob
import time
import getopt
import token
import tokenize
import operator

try:
    import kid.parser as kid_parser
except ImportError:
    kid_parser = None

__version__ = '1.5'

default_keywords = ['_']
DEFAULTKEYWORDS = ', '.join(default_keywords)

EMPTYSTRING = ''

# The normal pot-file header. msgmerge and Emacs's po-mode work better if it's
# there.
pot_header = _('''\
# SOME DESCRIPTIVE TITLE.
# Copyright (C) YEAR ORGANIZATION
# FIRST AUTHOR <EMAIL@ADDRESS>, YEAR.
#
msgid ""
msgstr ""
"Project-Id-Version: PACKAGE VERSION\\n"
"POT-Creation-Date: %(time)s\\n"
"PO-Revision-Date: YEAR-MO-DA HO:MI+ZONE\\n"
"Last-Translator: FULL NAME <EMAIL@ADDRESS>\\n"
"Language-Team: LANGUAGE <LL@li.org>\\n"
"MIME-Version: 1.0\\n"
"Content-Type: text/plain; charset=%(charset)s\\n"
"Content-Transfer-Encoding: %(charset)s\\n"
"Generated-By: pygettext.py %(version)s\\n"

''')

def usage(code, msg=''):
    print >> sys.stderr, __doc__ % globals()
    if msg:
        print >> sys.stderr, msg
    sys.exit(code)


escapes = []

def make_escapes(pass_iso8859):
    global escapes
    if pass_iso8859:
        # Allow iso-8859 characters to pass through so that e.g. 'msgid
        # "Höhe"' would result not result in 'msgid "H\366he"'.  Otherwise we
        # escape any character outside the 32..126 range.
        mod = 128
    else:
        mod = 256
    for i in range(256):
        if 32 <= (i % mod) <= 126:
            escapes.append(chr(i))
        else:
            escapes.append("\\%03o" % i)
    escapes[ord('\\')] = '\\\\'
    escapes[ord('\t')] = '\\t'
    escapes[ord('\r')] = '\\r'
    escapes[ord('\n')] = '\\n'
    escapes[ord('\"')] = '\\"'


def escape_ascii(s):
    """Escape all non-ascii text plus control chars and Python literals."""
    s = list(s)
    for i in range(len(s)):
        s[i] = escapes[ord(s[i])]
    return EMPTYSTRING.join(s)

def escape_unicode(s):
    """Escape control chars and Python literals, leave non-ascii text intact."""
    s = s.replace('\\', '\\\\').replace('\t', '\\t').replace(
        '\r', '\\r').replace('\n', '\\n').replace('\"', '\\"')
    # escape control chars
    def repl(m):
        return "\\%03o" % ord(m.group(0))
    return re.sub('[\001-\037]', repl, s)

def safe_eval(s):
    # unwrap quotes, safely
    return eval(s, {'__builtins__':{}}, {})


def normalize(s, escape=False):
    # This converts the various Python string types into a format that is
    # appropriate for .po files, namely much closer to C style.
    lines = s.split('\n')
    if len(lines) == 1:
        s = '"' + escape_unicode(s) + '"'
    else:
        if not lines[-1]:
            del lines[-1]
            lines[-1] = lines[-1] + '\n'
        for i in range(len(lines)):
            lines[i] = escape_unicode(lines[i])
        lineterm = '\\n"\n"'
        s = '""\n"' + lineterm.join(lines) + '"'
    if isinstance(s, unicode):
        s = s.encode('utf-8')
    if escape:
        def repl(m):
            return "\\%03o" % ord(m.group(0))
        s = re.sub('[\200-\377]', repl, s)
    return s

def containsAny(str, set):
    """Check whether 'str' contains ANY of the chars in 'set'"""
    return 1 in [c in str for c in set]


def _visit_pyfiles(list, dirname, names):
    """Helper for getFilesForName()."""
    # get extension for python source files
    if '_py_ext' not in globals():
        global _py_ext
        _py_ext = [triple[0] for triple in imp.get_suffixes()
                   if triple[2] == imp.PY_SOURCE][0]
    # don't recurse into CVS directories
    if 'CVS' in names:
        names.remove('CVS')
    if '.svn' in names:
        names.remove('.svn')
    # add all *.py files to list
    list.extend([os.path.join(dirname, file) for file in names
        if os.path.splitext(file)[1] == _py_ext])


def _get_modpkg_path(dotted_name, pathlist=None):
    """Get the filesystem path for a module or a package.

    Return the file system path to a file for a module, and to a directory for
    a package. Return None if the name is not found, or is a builtin or
    extension module.
    """
    # split off top-most name
    parts = dotted_name.split('.', 1)

    if len(parts) > 1:
        # we have a dotted path, import top-level package
        try:
            file, pathname, description = imp.find_module(parts[0], pathlist)
            if file:
                file.close()
        except ImportError:
            return None

        # check if it's indeed a package
        if description[2] == imp.PKG_DIRECTORY:
            # recursively handle the remaining name parts
            pathname = _get_modpkg_path(parts[1], [pathname])
        else:
            pathname = None
    else:
        # plain name
        try:
            file, pathname, description = imp.find_module(
                dotted_name, pathlist)
            if file:
                file.close()
            if description[2] not in [imp.PY_SOURCE, imp.PKG_DIRECTORY]:
                pathname = None
        except ImportError:
            pathname = None

    return pathname


def getFilesForName(name):
    """Get a list of module files for a filename, a module or package name,
    or a directory.
    """
    if not os.path.exists(name):
        # check for glob chars
        if containsAny(name, "*?[]"):
            files = glob.glob(name)
            list = []
            for file in files:
                list.extend(getFilesForName(file))
            return list

        # try to find module or package
        name = _get_modpkg_path(name)
        if not name:
            return []

    if os.path.isdir(name):
        # find all python files in directory
        list = []
        os.path.walk(name, _visit_pyfiles, list)
        return list
    elif os.path.exists(name):
        # a single file
        return [name]

    return []

class TokenEater:
    def __init__(self, options):
        self.__options = options
        self.__messages = {}
        self.__state = self.__waiting
        self.__data = []
        self.__lineno = -1
        self.__freshmodule = 1
        self.__curfile = None
        self.__encoding = None

    def __call__(self, ttype, tstring, stup, etup, line):
        # dispatch
        self.__state(ttype, tstring, stup[0])

    def __waiting(self, ttype, tstring, lineno):
        opts = self.__options
        # Do docstring extractions, if enabled
        if opts.docstrings and not opts.nodocstrings.get(self.__curfile):
            # module docstring?
            if self.__freshmodule:
                if ttype == tokenize.STRING:
                    self.__addentry(safe_eval(tstring), lineno, isdocstring=1)
                    self.__freshmodule = 0
                elif ttype not in (tokenize.COMMENT, tokenize.NL):
                    self.__freshmodule = 0
                return
            # class docstring?
            if ttype == tokenize.NAME and tstring in ('class', 'def'):
                self.__state = self.__suiteseen
                return
        if ttype == tokenize.NAME and tstring in opts.keywords:
            self.__state = self.__keywordseen

    def __suiteseen(self, ttype, tstring, lineno):
        # ignore anything until we see the colon
        if ttype == tokenize.OP and tstring == ':':
            self.__state = self.__suitedocstring

    def __suitedocstring(self, ttype, tstring, lineno):
        # ignore any intervening noise
        if ttype == tokenize.STRING:
            self.__addentry(safe_eval(tstring), lineno, isdocstring=1)
            self.__state = self.__waiting
        elif ttype not in (tokenize.NEWLINE, tokenize.INDENT,
                           tokenize.COMMENT):
            # there was no class docstring
            self.__state = self.__waiting

    def __keywordseen(self, ttype, tstring, lineno):
        if ttype == tokenize.OP and tstring == '(':
            self.__data = []
            self.__lineno = lineno
            self.__state = self.__openseen
        else:
            self.__state = self.__waiting

    def __openseen(self, ttype, tstring, lineno):
        if ttype == tokenize.OP and tstring == ',':
           # first handle case when the lang is explicitely set by the user
           if self.__data:
               self.__addentry(EMPTYSTRING.join(self.__data))
           self.__state = self.__waiting

        elif ttype == tokenize.OP and tstring == ')':
            # We've seen the last of the translatable strings.  Record the
            # line number of the first line of the strings and update the list
            # of messages seen.  Reset state for the next batch.  If there
            # were no strings inside _(), then just ignore this entry.
            if self.__data:
                self.__addentry(EMPTYSTRING.join(self.__data))
            self.__state = self.__waiting

        elif ttype == tokenize.STRING:
            self.__data.append(safe_eval(tstring))

        elif ttype not in [tokenize.COMMENT, token.INDENT, token.DEDENT,
                           token.NEWLINE, tokenize.NL]:
            # warn if we see anything else than STRING or whitespace
            print >> sys.stderr, _(
                '*** %(file)s:%(lineno)s: Seen unexpected token "%(token)s"'
                ) % {
                'token': tstring,
                'file': self.__curfile,
                'lineno': self.__lineno
                }
            self.__state = self.__waiting

    def __addentry(self, msg, lineno=None, isdocstring=0, iskidstring=0):
        # tokenize module always return unicode strings
        # even when they are in fact coded string instances
        # to deal with this we use a hack:
        # evaluate string's representation without leading "u"
        # to force interpration as coded string
        # then we decode it using already known file's encoding
        if not iskidstring:
            if type(msg) is str:
                msg = eval(repr(msg))
            else:
                msg = eval(repr(msg)[1:])
            msg = msg.decode(self.__encoding)
        if lineno is None:
            lineno = self.__lineno
        if not msg in self.__options.toexclude:
            entry = (self.__curfile, lineno)
            self.__messages.setdefault(msg, {})[entry] = isdocstring

    def set_filename(self, filename):
        self.__curfile = filename
        self.__freshmodule = 1

    def set_file_encoding(self, fp):
        """Search for -*- coding: -*- magic comment to find out file encoding."""
        self.__encoding = 'utf-8' # reset to default for each new file
        for line in fp.readlines()[:5]:
            m = re.match('#\s*-\*-\s+coding:\s+(\w+)\s+-\*-', line)
            if m:
                self.__encoding = m.group(1)
                break
        fp.seek(0)

    def __contains_inline_python(self, msg):
        return '${' in msg and not '$${' in msg

    def __strip_namespace_uri(self, tag):
        return tag.split('}', 1)[-1]

    def extract_kid_strings(self):
        if not self.__curfile:
            return
        if not kid_parser:
            raise ImportError, "Kid templating is not installed."
        tag = None
        tags = []
        for ev, item in kid_parser.document(self.__curfile):
            if ev == kid_parser.TEXT:
                if tag:
                    item = item.strip()
                    if item  and not self.__contains_inline_python(item):
                        self.__addentry(item, tag, iskidstring=1)
            elif ev == kid_parser.START:
                tag = item.tag
                if isinstance(tag, basestring):
                    tag = self.__strip_namespace_uri(tag)
                    if tag in ('script', 'style'):
                        tag = None
                else:
                    tag = None
                tags.append(tag)
            elif ev == kid_parser.END:
                if tags:
                    tag = tags.pop()

    def write(self, fp):
        options = self.__options
        # format without tz information
        # because %Z is timezone's name, not offset
        # and, say, on localized Windows XP this is non-ascii string
        timestamp = time.strftime('%Y-%m-%d %H:%M')
        # The time stamp in the header doesn't have the same format as that
        # generated by xgettext...
        t = {'time': timestamp, 'version': __version__, 'charset':'utf-8'}
        print >> fp, pot_header % t
        # Sort the entries.  First sort each particular entry's keys, then
        # sort all the entries by their first item.
        reverse = {}
        for k, v in self.__messages.items():
            keys = v.keys()
            keys.sort()
            reverse.setdefault(tuple(keys), []).append((k, v))
        rkeys = reverse.keys()
        rkeys.sort()
        for rkey in rkeys:
            rentries = reverse[rkey]
            rentries.sort()
            for k, v in rentries:
                isdocstring = 0
                # If the entry was gleaned out of a docstring, then add a
                # comment stating so.  This is to aid translators who may wish
                # to skip translating some unimportant docstrings.
                if reduce(operator.__add__, v.values()):
                    isdocstring = 1
                # k is the message string, v is a dictionary-set of (filename,
                # lineno) tuples.  We want to sort the entries in v first by
                # file name and then by line number.
                v = v.keys()
                v.sort()
                if not options.writelocations:
                    pass
                # location comments are different b/w Solaris and GNU:
                elif options.locationstyle == options.SOLARIS:
                    for filename, lineno in v:
                        d = {'filename': filename, 'lineno': lineno}
                        print >> fp, _(
                            '# File: %(filename)s, line: %(lineno)s') % d
                elif options.locationstyle == options.GNU:
                    # fit as many locations on one line, as long as the
                    # resulting line length doesn't exceeds 'options.width'
                    locline = '#:'
                    for filename, lineno in v:
                        d = {'filename': filename, 'lineno': lineno}
                        s = _(' %(filename)s:%(lineno)s') % d
                        if len(locline) + len(s) <= options.width:
                            locline += s
                        else:
                            print >> fp, locline
                            locline = "#:" + s
                    if len(locline) > 2:
                        print >> fp, locline
                if isdocstring:
                    print >> fp, '#, docstring'
                if k: # do not output empty msgid
                    print >> fp, 'msgid', normalize(k, options.escape)
                    print >> fp, 'msgstr ""\n'

def main():
    global default_keywords
    try:
        opts, args = getopt.getopt(
            sys.argv[1:],
            'ad:UDEhk:Kno:p:S:Vvw:x:X:',
            ['extract-all', 'default-domain=', 'escape', 'help',
             'keyword=', 'no-default-keywords',
             'add-location', 'no-location', 'output=', 'output-dir=',
             'style=', 'verbose', 'version', 'width=', 'exclude-file=',
             'docstrings', 'no-docstrings', 'support-unicode',
             ])
    except getopt.error, msg:
        usage(1, msg)

    # for holding option values
    class Options:
        # constants
        GNU = 1
        SOLARIS = 2
        # defaults
        extractall = 0 # FIXME: currently this option has no effect at all.
        escape = 0
        keywords = []
        outpath = ''
        outfile = 'messages.pot'
        writelocations = 1
        locationstyle = GNU
        verbose = 0
        width = 78
        excludefilename = ''
        docstrings = 0
        nodocstrings = {}

    options = Options()
    locations = {'gnu' : options.GNU,
                 'solaris' : options.SOLARIS,
                 }

    # parse options
    for opt, arg in opts:
        if opt in ('-h', '--help'):
            usage(0)
        elif opt in ('-a', '--extract-all'):
            options.extractall = 1
        elif opt in ('-d', '--default-domain'):
            options.outfile = arg + '.pot'
        elif opt in ('-E', '--escape'):
            options.escape = 1
        elif opt in ('-D', '--docstrings'):
            options.docstrings = 1
        elif opt in ('-k', '--keyword'):
            options.keywords.append(arg)
        elif opt in ('-K', '--no-default-keywords'):
            default_keywords = []
        elif opt in ('-n', '--add-location'):
            options.writelocations = 1
        elif opt in ('--no-location',):
            options.writelocations = 0
        elif opt in ('-S', '--style'):
            options.locationstyle = locations.get(arg.lower())
            if options.locationstyle is None:
                usage(1, _('Invalid value for --style: %s') % arg)
        elif opt in ('-o', '--output'):
            options.outfile = arg
        elif opt in ('-p', '--output-dir'):
            options.outpath = arg
        elif opt in ('-v', '--verbose'):
            options.verbose = 1
        elif opt in ('-V', '--version'):
            print _('pygettext.py (xgettext for Python) %s') % __version__
            sys.exit(0)
        elif opt in ('-w', '--width'):
            try:
                options.width = int(arg)
            except ValueError:
                usage(1, _('--width argument must be an integer: %s') % arg)
        elif opt in ('-x', '--exclude-file'):
            options.excludefilename = arg
        elif opt in ('-X', '--no-docstrings'):
            fp = open(arg)
            try:
                while 1:
                    line = fp.readline()
                    if not line:
                        break
                    options.nodocstrings[line[:-1]] = 1
            finally:
                fp.close()

    # calculate escapes
    make_escapes(0)

    # calculate all keywords
    options.keywords.extend(default_keywords)

    # initialize list of strings to exclude
    if options.excludefilename:
        try:
            fp = open(options.excludefilename)
            options.toexclude = fp.readlines()
            fp.close()
        except IOError:
            print >> sys.stderr, _(
                "Can't read --exclude-file: %s") % options.excludefilename
            sys.exit(1)
    else:
        options.toexclude = []

    # resolve args to module lists
    expanded = []
    for arg in args:
        if arg == '-':
            expanded.append(arg)
        else:
            expanded.extend(getFilesForName(arg))
    args = expanded

    # slurp through all the files
    eater = TokenEater(options)
    for filename in args:
        if filename == '-':
            if options.verbose:
                print _('Reading standard input')
            fp = sys.stdin
            closep = 0
        else:
            if options.verbose:
                print _('Working on %s') % filename
            fp = open(filename)
            eater.set_file_encoding(fp)
            closep = 1
        try:
            eater.set_filename(filename)
            if os.path.splitext(filename)[-1].lower() == '.kid':
                try:
                    eater.extract_kid_strings()
                except Exception, e:
                    print >> sys.stderr, e
            else:
                try:
                    tokenize.tokenize(fp.readline, eater)
                except tokenize.TokenError, e:
                    print >> sys.stderr, '%s: %s, line %d, column %d' % (
                        e[0], filename, e[1][0], e[1][1])
        finally:
            if closep:
                fp.close()

    # write the output
    if options.outfile == '-':
        fp = sys.stdout
        closep = 0
    else:
        if options.outpath:
            options.outfile = os.path.join(options.outpath, options.outfile)
        fp = open(options.outfile, 'wt')
        closep = 1
    try:
        eater.write(fp)
    finally:
        if closep:
            fp.close()

if __name__ == '__main__':
    main()
    # some more test strings
    _(u'a unicode string')
    # this one creates a warning
    _('*** Seen unexpected token "%(token)s"') % {'token': 'test'}
    _('more' 'than' 'one' 'string')




import sys
from StringIO import StringIO

from turbogears import config, util


def test_deprecated():

    def some_old_function(x, y):
        """some old function"""
        return x + y
    some_old_function = util.deprecated('this is old')(some_old_function)

    assert "some old function" in some_old_function.__doc__
    some_old_function.__name__ == 'some_old_function'

    stderr, sys.stderr = sys.stderr, StringIO()
    try:
        assert some_old_function(1, 2) == 3
    finally:
        stderr, sys.stderr = sys.stderr, stderr
    assert 'DeprecationWarning: this is old' in stderr.getvalue()
    stderr.close()


def test_adapt_call():
    adapt = util.adapt_call

    def f(a, b, x=1, y=2):
        pass

    assert adapt(f, (), dict()) == ((), dict())
    assert adapt(f, (1,), dict(a=1)) == ((), dict(a=1))
    assert adapt(f, (1,), dict(b=2)) == ((1,), dict(b=2))
    assert adapt(f, (1,), dict(c=3)) == ((1,), dict())
    assert adapt(f, (1,), dict(y=4)) == ((1,), dict(y=4))
    assert adapt(f, (1, 2, 3), dict()) == ((1, 2, 3), dict())
    assert adapt(f, (1, 2, 3), dict(x=3)) == ((1, 2), dict(x=3))
    assert adapt(f, (1, 2, 3), dict(y=4)) == ((1, 2, 3), dict(y=4))
    assert adapt(f, (1, 2, 3), dict(z=5)) == ((1, 2, 3), dict())
    assert adapt(f, (1, 2, 3, 4), dict()) == ((1, 2, 3, 4), dict())
    assert adapt(f, (1, 2, 3, 4, 5), dict()) == ((1, 2, 3, 4), dict())
    assert adapt(f, (1, 2, 3, 4, 5, 6), dict()) == ((1, 2, 3, 4), dict())
    assert adapt(f, (1, 2, 3), dict(y=4, z=5)) == ((1, 2, 3), dict(y=4))

    def f(a, b, x=1, y=2, *args):
        pass

    assert adapt(f, (), dict()) == ((), dict())
    assert adapt(f, (1,), dict(a=1)) == ((), dict(a=1))
    assert adapt(f, (1,), dict(b=2)) == ((1,), dict(b=2))
    assert adapt(f, (1,), dict(c=3)) == ((1,), dict())
    assert adapt(f, (1,), dict(y=4)) == ((1,), dict(y=4))
    assert adapt(f, (1, 2, 3), dict()) == ((1, 2, 3), dict())
    assert adapt(f, (1, 2, 3), dict(x=3)) == ((1, 2), dict(x=3))
    assert adapt(f, (1, 2, 3), dict(y=4)) == ((1, 2, 3), dict(y=4))
    assert adapt(f, (1, 2, 3), dict(z=5)) == ((1, 2, 3), dict())
    assert adapt(f, (1, 2, 3, 4), dict()) == ((1, 2, 3, 4), dict())
    assert adapt(f, (1, 2, 3, 4, 5), dict()) == ((1, 2, 3, 4, 5), dict())
    assert adapt(f, (1, 2, 3, 4, 5, 6), dict()) == ((1, 2, 3, 4, 5, 6), dict())
    assert adapt(f, (1, 2, 3), dict(y=4, z=5)) == ((1, 2, 3), dict(y=4))

    def f(a, b, x=1, y=2, **kw):
        pass

    assert adapt(f, (), dict()) == ((), dict())
    assert adapt(f, (1,), dict(a=1)) == ((), dict(a=1))
    assert adapt(f, (1,), dict(b=2)) == ((1,), dict(b=2))
    assert adapt(f, (1,), dict(c=3)) == ((1,), dict(c=3))
    assert adapt(f, (1,), dict(y=4)) == ((1,), dict(y=4))
    assert adapt(f, (1, 2, 3), dict()) == ((1, 2, 3), dict())
    assert adapt(f, (1, 2, 3), dict(x=3)) == ((1, 2), dict(x=3))
    assert adapt(f, (1, 2, 3), dict(y=4)) == ((1, 2, 3), dict(y=4))
    assert adapt(f, (1, 2, 3), dict(z=5)) == ((1, 2, 3), dict(z=5))
    assert adapt(f, (1, 2, 3, 4), dict()) == ((1, 2, 3, 4), dict())
    assert adapt(f, (1, 2, 3, 4, 5), dict()) == ((1, 2, 3, 4), dict())
    assert adapt(f, (1, 2, 3, 4, 5, 6), dict()) == ((1, 2, 3, 4), dict())
    assert adapt(f, (1, 2, 3), dict(y=4, z=5)) == ((1, 2, 3), dict(y=4, z=5))

    def f(a, b, x=1, y=2, *args, **kwargs):
        pass

    assert adapt(f, (), dict()) == ((), dict())
    assert adapt(f, (1,), dict(a=1)) == ((), dict(a=1))
    assert adapt(f, (1,), dict(b=2)) == ((1,), dict(b=2))
    assert adapt(f, (1,), dict(c=3)) == ((1,), dict(c=3))
    assert adapt(f, (1,), dict(y=4)) == ((1,), dict(y=4))
    assert adapt(f, (1, 2, 3), dict()) == ((1, 2, 3), dict())
    assert adapt(f, (1, 2, 3), dict(x=3)) == ((1, 2), dict(x=3))
    assert adapt(f, (1, 2, 3), dict(y=4)) == ((1, 2, 3), dict(y=4))
    assert adapt(f, (1, 2, 3), dict(z=5)) == ((1, 2, 3), dict(z=5))
    assert adapt(f, (1, 2, 3, 4), dict()) == ((1, 2, 3, 4), dict())
    assert adapt(f, (1, 2, 3, 4, 5), dict()) == ((1, 2, 3, 4, 5), dict())
    assert adapt(f, (1, 2, 3, 4, 5, 6), dict()) == ((1, 2, 3, 4, 5, 6), dict())
    assert adapt(f, (1, 2, 3), dict(y=4, z=5)) == ((1, 2, 3), dict(y=4, z=5))


def test_call_on_stack():

    def recurse(level):
        if level:
            return recurse(level-1)
        return util.call_on_stack('recurse', dict())

    assert not recurse(0)
    assert recurse(1)

    def recurse(level, arg=None):
        if level:
            return recurse(level-1, arg)
        return util.call_on_stack('recurse', dict(arg='true'), 3)

    assert not recurse(3, 'true')
    assert recurse(4, 'true')
    assert not recurse(4, 'false')


def test_remove_keys():

    assert util.remove_keys(dict(x=1, y=2, z=3), 'xz') == dict(y=2)


def test_has_arg():

    def fun(x, z=2):
        pass

    assert util.has_arg(fun, 'x')
    assert not util.has_arg(fun, 'y')
    assert util.has_arg(fun, 'z')


def test_arg_index():

    def fun(a, b, y=3, z=4):
        pass

    assert util.arg_index(fun, 'a') == 0
    assert util.arg_index(fun, 'b') == 1
    assert util.arg_index(fun, 'c') is None
    assert util.arg_index(fun, 'y') == 2
    assert util.arg_index(fun, 'z') == 3


def test_parse_http_accept_header():
    t = util.parse_http_accept_header
    assert t(None) == []
    assert t('') == []
    assert t('text/html') == ['text/html']
    assert t('text/html;level=1') == ['text/html;level=1']
    assert t(' text/html; level=1 ') == ['text/html; level=1']
    assert t('audio/*, audio/mid') == ['audio/*', 'audio/mid']
    assert t('audio/*; q=0.2, audio/mid') == ['audio/mid', 'audio/*']
    assert t('audio/*; q=0.2, audio/mid; q=0.3') == ['audio/mid', 'audio/*']
    assert t('audio/*; q=0.3, audio/mid; q=0.2') == ['audio/*', 'audio/mid']
    assert t('audio/*, audio/mid; q=0.2') == ['audio/*', 'audio/mid']
    assert t('audio/*, audio/mid; q=0') == ['audio/*']
    assert t('''text/plain; q=0.5, text/html,
        text/x-dvi; q=0.8, text/x-c''') == ['text/html', 'text/x-c',
            'text/x-dvi', 'text/plain']
    assert t('''text/*;q=0.3, text/html;q=0.7, text/html;level=1,
        text/html;level=2;q=0.4, */*;q=0.5''') == ['text/html;level=1',
            'text/html', '*/*', 'text/html;level=2', 'text/*']
    assert t('compress;q=0.5, gzip;q=1.0') == ['gzip', 'compress']
    assert t('gzip;q=1.0, identity; q=0.5, *;q=0') == ['gzip', 'identity']
    assert t('da, en-gb;q=0.8, en;q=0.7') == ['da', 'en-gb', 'en']
    assert t('da, en-gb;q=0.7, en;q=0.8') == ['da', 'en', 'en-gb']
    assert t('da;q=0.75, en-gb;q=0.099, en;q=0.8') == ['en', 'da', 'en-gb']


def test_preferred_from_http_accept_header():
    t = util.simplify_http_accept_header
    assert t(None) is None
    assert t('') == ''
    assert t('text/html') == 'text/html'
    assert t('text/html;level=1') == 'text/html'
    assert t(' text/html; level=1 ') == 'text/html'
    assert t('audio/*, audio/mid') == 'audio/*'
    assert t('audio/*; q=0.2, audio/mid') == 'audio/mid'
    assert t('audio/*; q=0.2, audio/mid; q=0.3') == 'audio/mid'
    assert t('audio/*; q=0.3, audio/mid; q=0.2') == 'audio/*'
    assert t('audio/*, audio/mid; q=0.2') == 'audio/*'
    assert t('audio/*, audio/mid; q=0') == 'audio/*'
    assert t('''text/plain; q=0.5, text/html,
        text/x-dvi; q=0.8, text/x-c''') == 'text/html'
    assert t('''text/*;q=0.3, text/html;q=0.7, text/html;level=1,
        text/html;level=2;q=0.4, */*;q=0.5''') == 'text/html'
    assert t('''text/*;q=0.3, text/plain;q=0.7, text/html;level=1,
        text/sgml;q=0.4, */*;q=0.5''') == 'text/html'
    assert t('''text/*;q=0.3, text/plain;q=0.7, text/html;level=1;q=0.6,
        text/sgml;q=0.4, */*;q=0.5''') == 'text/plain'
    assert t('compress;q=0.5, gzip;q=1.0') == 'gzip'
    assert t('gzip;q=1.0, identity; q=0.5, *;q=0') == 'gzip'
    assert t('da, en-gb;q=0.8, en;q=0.7') == 'da'
    assert t('da, en-gb;q=0.7, en;q=0.8') == 'da'
    assert t('da;q=0.75, en-gb;q=0.099, en;q=0.8') == 'en'


def test_quote_cookie():
    assert util.quote_cookie ('Hello, W\xf6rld') \
        == 'Hello%2C%20W\xf6rld'
    assert util.quote_cookie ('$1;\tinsert coin!') \
        == '%241%3B%09insert%20coin!'


def test_unquote_cookie():
    assert util.unquote_cookie ('Hello%2C%20W\xf6rld!') \
        == 'Hello, W\xf6rld!'
    assert util.unquote_cookie ('%241%3B%09insert%20coin!') \
        == '$1;\tinsert coin!'


def test_get_template_encoding_default():
    assert util.get_template_encoding_default() == 'utf-8'


def test_get_mime_type_for_format():
    assert util.get_mime_type_for_format('plain') == 'text/plain'
    assert util.get_mime_type_for_format('text') == 'text/plain'
    assert util.get_mime_type_for_format('html') == 'text/html'
    assert util.get_mime_type_for_format('xhtml') == 'text/html'
    assert util.get_mime_type_for_format('xml') == 'text/xml'
    assert util.get_mime_type_for_format('json') == 'application/json'
    assert util.get_mime_type_for_format('foo') == 'text/html'
    config.update({'global': {'tg.format_mime_types':
        {'xhtml': 'application/xhtml+xml', 'foo': 'text/bar'}}})
    assert util.get_mime_type_for_format('xhtml') == 'application/xhtml+xml'
    assert util.get_mime_type_for_format('json') == 'application/json'
    assert util.get_mime_type_for_format('foo') == 'text/bar'
    config.update({'global': {'tg.format_mime_types': {}}})
    assert util.get_mime_type_for_format('xhtml') == 'text/html'
    assert util.get_mime_type_for_format('foo') == 'text/html'


def test_mime_type_has_charset():
    assert not util.mime_type_has_charset(None)
    assert not util.mime_type_has_charset('foo/bar')
    assert util.mime_type_has_charset('text/foobar')
    assert util.mime_type_has_charset('application/xml')
    assert util.mime_type_has_charset('application/foo+xml')
    assert util.mime_type_has_charset('application/javascript')
    assert not util.mime_type_has_charset('application/foobar')
    assert not util.mime_type_has_charset('application/json')


def test_find_precision():
    assert util.find_precision(0) == 0
    assert util.find_precision(42) == 0
    assert util.find_precision(0.1) == 1
    assert util.find_precision(1234567.8) == 1
    assert util.find_precision(34.25) == 2
    assert util.find_precision(1234567.123) == 3
    assert util.find_precision(123.1234567) == 7


def test_copy_if_mutable():
    test_values = ((None, False),
        ('foo', False), (42, False), ((1, 2), False),
        ([1, 2], True), ({1: 2}, True))
    for value, mutable in test_values:
        if mutable:
            assert util.copy_if_mutable(value) == value
            assert util.copy_if_mutable(value) is not value
            ret = util.copy_if_mutable(value, True)
            assert ret[0] == value
            assert ret[0] is not value
            assert ret[1] is True
        else:
            assert util.copy_if_mutable(value) is value
            ret = util.copy_if_mutable(value, True)
            assert ret[0] is value
            assert ret[1] is False


def test_fixentities():
    assert util.fixentities('Chip&nbsp;&amp;&nbsp;Chap') \
        == 'Chip&#160;&amp;&#160;Chap'
    assert util.fixentities('<&quot;&lt;&copy;&gt;&quot;>') \
        == '<&quot;&lt;&#169;&gt;&quot;>'



import turbogears
from turbogears import controllers, expose
import sys
import StringIO
from code import InteractiveConsole


class WebConsole(controllers.RootController):
    """Web based Python interpreter"""

    __label__ = 'WebConsole'

    icon = "/tg_static/images/shell.png"

    def __init__(self, width=80):
        self.console = None

        try:
            sys.ps1
        except AttributeError:
            sys.ps1 = '>>> '
        try:
            sys.ps2
        except AttributeError:
            sys.ps2 = '... '

    def process_request(self, line):
        more, output = self._process_request(line)
        return dict(more=more, output=output)
    process_request = expose()(process_request)

    def process_multiline_request(self, block):
        outlines = []

        lines = [line for line in block.split('\n')]

        for line in lines:
            more, output = self._process_request(line)

            if output[-1] == '\n':     # we'll handle the newlines later.
                output = output[:-1]

            outlines.append(output)

        return dict(more=more, output='\n'.join(outlines))
    process_multiline_request = expose()(process_multiline_request)

    def _process_request(self, line):
        if len(self.console.buffer):
            prompt = sys.ps2
        else:
            prompt = sys.ps1

        myout = StringIO.StringIO()

        output = "%s%s" % (prompt, line)
        # hopefully python doesn't interrupt in this block lest we'll get some curious output.
        try:
            sys.stdout = myout
            sys.stderr = myout
            more = self.console.push(line)
        finally:
            sys.stdout = sys.__stdout__
            sys.stderr = sys.__stderr__

        stdout = myout.getvalue()

        if stdout:
            output = '%s\n%s' % (output, stdout)

        return more, output.rstrip()

    def new_console(self):
        locs = dict(__name__='tg-admin', __doc__=None,
            reload_console=self.new_console)
        mod = turbogears.util.get_model()
        if mod:
            locs.update(mod.__dict__)
        self.console = InteractiveConsole(locals=locs)

    def index(self):
        if not self.console:
            self.new_console()
        return dict()
    index = expose(template="turbogears.toolbox.console")(index)



# Formatting configuration for locale gl_ES

languages={'gl': 'galego'}

countries={'ES': u'Espa\xf1a'}

months=['Xaneiro', 'Febreiro', 'Marzo', 'Abril', 'Maio', u'Xu\xf1o', 'Xullo', 'Agosto', 'Setembro', 'Outubro', 'Novembro', 'Decembro']

abbrMonths=['Xan', 'Feb', 'Mar', 'Abr', 'Mai', u'Xu\xf1', 'Xul', 'Ago', 'Set', 'Out', 'Nov', 'Dec']

days=['Luns', 'Martes', u'M\xe9rcores', 'Xoves', 'Venres', u'S\xe1bado', 'Domingo']

abbrDays=['Lun', 'Mar', u'M\xe9r', 'Xov', 'Ven', u'S\xe1b', 'Dom']

dateFormats={'medium': '%%(abbrmonthname)s %d,%y', 'full': '%%(dayname)s %d %%(monthname)s %Y', 'long': '%d %%(monthname)s %Y', 'short': '%d/%m/%y'}

numericSymbols={'group': '.', 'nativeZeroDigit': '0', 'exponential': 'E', 'perMille': u'\u2030', 'nan': u'\ufffd', 'decimal': ',', 'percentSign': '%', 'list': ';', 'patternDigit': '#', 'plusSign': '+', 'infinity': u'\u221e', 'minusSign': '-'}



"""Commands for the TurboGears command line tool."""

import optparse
import sys
import os
import os.path
import glob

import pkg_resources
import dispatch

import configobj

import turbogears
from turbogears.util import get_model, load_project_config, \
        get_project_config, get_package_name
from turbogears.identity import SecureObject, from_any_host
from turbogears import config, database
from sacommand import sacommand

sys.path.insert(0, os.getcwd())

no_connection_param = ["help", "list"]
no_model_param = ["help"]


def silent_os_remove(fname):
    """Try to remove file 'fname' but mute any error that may happen.

    Returns True if file was actually removed and False otherwise.

    """
    try:
        os.remove(fname)
        return True
    except os.error:
        pass
    return False


class CommandWithDB(object):
    """Base class for commands that need to use the database"""

    config = None

    def __init__(self, version):
        pass

    def find_config(self):
        """Chooses the config file, trying to guess whether this is a
        development or installed project."""
        load_project_config(self.config)
        self.dburi = config.get("sqlobject.dburi", None)
        if self.dburi and self.dburi.startswith("notrans_"):
            self.dburi = self.dburi[8:]


class SQL(CommandWithDB):
    """Wrapper command for sqlobject-admin, and some sqlalchemy support.

    This automatically supplies sqlobject-admin with the database that
    is found in the config file.

    Will also supply the model module as appropriate.

    """

    desc = "Run the database provider manager"
    need_project = True

    def __init__(self, version):
        if len(sys.argv) == 1 or sys.argv[1][0] == "-":
            parser = optparse.OptionParser(
                usage="%prog sql [command]\n\n" \
                      "hint: '%prog sql help' will list the sqlobject " \
                      "commands",
                version="%prog " + version)
            parser.add_option("-c", "--config", help="config file",
                              dest="config")
            options, args = parser.parse_args(sys.argv[1:3])

            if not options.config:
                parser.error("Please provide a valid option or command.")
            self.config = options.config
            # get rid of our config option
            if args:
                del sys.argv[1]
            else:
                del sys.argv[1:3]

        self.find_config()

    def run(self):
        """Executes the sqlobject-admin code."""
        if not "--egg" in sys.argv and not turbogears.util.get_project_name():
            print "This don't look like a TurboGears project."
            return
        else:
            command = sys.argv[1]

            if config.get("sqlalchemy.dburi"):
                try:
                    sacommand(command, sys.argv)
                except dispatch.interfaces.NoApplicableMethods:
                    sacommand("help", [])
                return

            try:
                from sqlobject.manager import command as socommand
            except ImportError:
                print "Error: SQLObject not installed."
                return

            if command not in no_connection_param:
                if self.dburi:
                    print "Using database URI %s" % self.dburi
                    sys.argv.insert(2, self.dburi)
                    sys.argv.insert(2, "-c")
                else:
                    print ("Database URI not specified in the config file"
                        " (%s).\nPlease be sure it's on the command line."
                            % (self.config or get_project_config()))

            if command not in no_model_param:
                if not "--egg" in sys.argv:
                    eggname = glob.glob("*.egg-info")
                    if not eggname or not os.path.exists(
                            os.path.join(eggname[0], "sqlobject.txt")):
                        eggname = self.fix_egginfo(eggname)
                    eggname = eggname[0].replace(".egg-info", "")
                    if not "." in sys.path:
                        sys.path.append(".")
                        pkg_resources.working_set.add_entry(".")
                    sys.argv.insert(2, eggname)
                    sys.argv.insert(2, "--egg")

            socommand.the_runner.run(sys.argv)

    def fix_egginfo(self, eggname):
        """Add egg-info directory if necessary."""
        print """
This project seems incomplete. In order to use the sqlobject commands
without manually specifying a model, there needs to be an
egg-info directory with an appropriate sqlobject.txt file.

I can fix this automatically. Would you like me to?
"""
        dofix = raw_input("Enter [y] or n: ")
        if not dofix or dofix.lower()[0] == 'y':
            oldargs = sys.argv
            sys.argv = ["setup.py", "egg_info"]
            import imp
            imp.load_module("setup", *imp.find_module("setup", ["."]))
            sys.argv = oldargs

            import setuptools
            package = setuptools.find_packages()[0]
            eggname = glob.glob("*.egg-info")
            sqlobjectmeta = open(os.path.join(eggname[0], "sqlobject.txt"), "w")
            sqlobjectmeta.write("""db_module=%(package)s.model
history_dir=$base/%(package)s/sqlobject-history
""" % dict(package=package))
        else:
            sys.exit(0)
        return eggname


class Shell(CommandWithDB):
    """Convenient version of the Python interactive shell.

    This shell attempts to locate your configuration file and model module
    so that it can import everything from your model and make it available
    in the Python shell namespace.

    """

    desc = "Start a Python prompt with your database available"
    need_project = True

    def run(self):
        """Run the shell"""
        self.find_config()

        mod = get_model()
        if mod:
            locals = mod.__dict__
        else:
            locals = dict(__name__="tg-admin")

        if config.get("sqlalchemy.dburi"):
            using_sqlalchemy = True
            database.get_engine()
            locals.update(dict(session=database.session,
                metadata=database.metadata))
        else:
            using_sqlalchemy = False

        try:
            # try to use IPython if possible
            import IPython

            class CustomIPShell(IPython.iplib.InteractiveShell):
                def raw_input(self, *args, **kw):
                    try:
                        return IPython.iplib.InteractiveShell.raw_input(self,
                            *args, **kw) # needs decoding (see below)?
                    except EOFError:
                        r = raw_input("Do you wish to commit your "
                                    "database changes? [yes]")
                        if not r.lower().startswith("n"):
                            if using_sqlalchemy:
                                self.push("session.flush()")
                            else:
                                self.push("hub.commit()")
                        raise EOFError

            shell = IPython.Shell.IPShell(user_ns=locals,
                                          shell_class=CustomIPShell)
            shell.mainloop()
        except ImportError:
            import code

            class CustomShell(code.InteractiveConsole):
                def raw_input(self, *args, **kw):
                    try:
                        import readline
                    except ImportError:
                        pass
                    try:
                        r = code.InteractiveConsole.raw_input(self,
                            *args, **kw)
                        for encoding in (getattr(sys.stdin, 'encoding', None),
                                sys.getdefaultencoding(), 'utf-8', 'latin-1'):
                            if encoding:
                                try:
                                    return r.decode(encoding)
                                except UnicodeError:
                                    pass
                        return r
                    except EOFError:
                        r = raw_input("Do you wish to commit your "
                                      "database changes? [yes]")
                        if not r.lower().startswith("n"):
                            if using_sqlalchemy:
                                self.push("session.flush()")
                            else:
                                self.push("hub.commit()")
                        raise EOFError

            shell = CustomShell(locals=locals)
            shell.interact()


class ToolboxCommand(CommandWithDB):

    desc = "Launch the TurboGears Toolbox"

    def __init__(self, version):
        self.hostlist = ['127.0.0.1', '::1', '::ffff:127.0.0.1']

        parser = optparse.OptionParser(
            usage="%prog toolbox [options]",
            version="%prog " + version)
        parser.add_option("-n", "--no-open",
            help="don't open browser automatically",
            dest="noopen", action="store_true", default=False)
        parser.add_option("-c", "--add-client",
            help="allow client ip address specified to connect to toolbox"
                " (can be specified more than once)",
            dest="host", action="append", default=None)
        parser.add_option("-p", "--port",
            help="port to run the Toolbox on",
            dest="port", default=7654)
        parser.add_option("--config",
            help="config file", dest="config",
            default=self.config or get_project_config())

        options, args = parser.parse_args(sys.argv[1:])
        self.port = int(options.port)
        self.noopen = options.noopen
        self.config = options.config

        if options.host:
            self.hostlist = self.hostlist + options.host

        turbogears.widgets.load_widgets()

    def openbrowser(self):
        import webbrowser
        webbrowser.open("http://localhost:%d" % self.port)

    def run(self):
        import cherrypy
        from turbogears import toolbox

        # Make sure we have full configuration with every option
        # in it so other plugins or whatever find what they need
        # when starting even inside the toolblox
        conf = get_package_name()
        conf = conf and "%s.config" % conf or None
        conf = config.config_obj(configfile=self.config, modulename=conf)

        if 'global' in conf:
            config.update({'global': conf['global']})

        root = SecureObject(toolbox.Toolbox(), from_any_host(self.hostlist),
                exclude=['noaccess'])

        cherrypy.tree.mount(root, "/")

        # amend some parameters since we are running from the command
        # line in order to change port, log methods...
        config.update({'global': {
            'server.socket_port': self.port,
            'server.webpath': '/',
            'server.environment': 'development',
            'server.log_to_screen': True,
            'autoreload.on': False,
            'server.package': 'turbogears.toolbox',
            'log_debug_info_filter.on': False,
            'identity.failure_url': '/noaccess',
            'identity.force_external_redirect': False,
            'tg.defaultview': 'kid',
            'tg.strict_parameters': False,
            'kid.outputformat': 'html default',
            'kid.encoding': 'utf-8'
            }})

        if not self.noopen:
            cherrypy.server.start_with_callback(self.openbrowser)
        else:
            cherrypy.server.start()


commands = None

def main():
    """Main command runner. Manages the primary command line arguments."""
    # add commands defined by entrypoints
    commands = {}
    for entrypoint in pkg_resources.iter_entry_points("turbogears.command"):
        command = entrypoint.load()
        commands[entrypoint.name] = (command.desc, entrypoint)

    def _help():
        """Custom help text for tg-admin."""

        print """
TurboGears %s command line interface

Usage: %s [options] <command>

Options:
    -c CONFIG --config=CONFIG    Config file to use
    -e EGG_SPEC --egg=EGG_SPEC   Run command on given Egg

Commands:""" % (turbogears.__version__, sys.argv[0])

        longest = max([len(key) for key in commands.keys()])
        format = "%" + str(longest) + "s  %s"
        commandlist = commands.keys()
        commandlist.sort()
        for key in commandlist:
            print format % (key, commands[key][0])

    parser = optparse.OptionParser()
    parser.allow_interspersed_args = False
    parser.add_option("-c", "--config", dest="config")
    parser.add_option("-e", "--egg", dest="egg")
    parser.print_help = _help
    options, args = parser.parse_args(sys.argv[1:])

    # if command is not found display help
    if not args or not commands.has_key(args[0]):
        _help()
        sys.exit()

    commandname = args[0]
    # strip command and any global options from the sys.argv
    sys.argv = [sys.argv[0]] + args[1:]
    command = commands[commandname][1]
    command = command.load()

    if options.egg:
        egg = pkg_resources.get_distribution(options.egg)
        os.chdir(egg.location)

    if hasattr(command,"need_project"):
        if not turbogears.util.get_project_name():
            print "This command needs to be run from inside a project directory"
            return
        elif not options.config and not os.path.isfile(get_project_config()):
            print """No default config file was found.
If it has been renamed use:
tg-admin --config=<FILE> %s""" % commandname
            return
    command.config = options.config
    command = command(turbogears.__version__)
    command.run()


__all__ = ["main"]



# -*- coding: utf-8 -*-

from turbogears import view, config
import unittest


class TestView(unittest.TestCase):

    def setUp(self):
        if not view.engines:
            view.load_engines()

    def test_cycle(self):
        oe = view.base.cycle(('odd','even'))
        assert str(oe) == str(None)
        assert oe.next() == 'odd'
        assert str(oe) == 'odd'
        assert oe.next() == 'even'
        assert oe.value == 'even'
        assert oe.next() == 'odd'
        assert oe.value == 'odd'

    def test_selector(self):
        assert view.base.selector(False) is None
        assert view.base.selector(True) == 'selected'

    def test_checker(self):
        assert view.base.checker(False) is None
        assert view.base.checker(True) == 'checked'

    def test_ipeek(self):
        seq = xrange(3, 6)
        assert view.base.ipeek(seq)
        assert list(seq) == range(3, 6)
        seq = xrange(3, 3)
        assert not view.base.ipeek(seq)
        assert list(seq) == []

    def test_UnicodeValueAppearingInATemplateIsFine(self):
        ustr = u"micro-eXtreme Programming ( µ XP): Embedding XP Within Standard Projects"
        info = dict(someval=ustr)
        val = view.render(info, template="turbogears.tests.simple")
        self.failUnless(u"Paging all " + ustr in val.decode("utf-8"))

    def test_templateRetrievalByPath(self):
        config.update({'server.environment' : 'development'})
        from turbokid import kidsupport
        ks = kidsupport.KidSupport()
        cls = ks.load_template("turbogears.tests.simple")
        assert cls is not None
        t = cls()
        t.someval = "hello"
        filled = str(t)
        assert "groovy" in filled
        assert "html" in filled
        # the following import should not fail, if everything is working correctly:
        import turbogears.tests.simple

    def test_default_output_encoding(self):
        info = dict(someval="someval")
        template = "turbogears.tests.simple"
        headers = {}
        # default encoding is utf-8
        val = view.render(info, template, headers=headers)
        assert headers.get('Content-Type') == 'text/html; charset=utf-8'
        # encoding can be changed and will be added to existing content type
        try:
            config.update({'kid.encoding': 'iso-8859-1'})
            headers['Content-Type'] = 'text/html'
            view.render(info, template, headers=headers)
            assert headers.get('Content-Type') == 'text/html; charset=iso-8859-1'
        finally:
            config.update({'kid.encoding': 'utf-8'})

    def test_content_types(self):
        info = dict(someval="someval")
        template = "turbogears.tests.simple"
        headers = {}
        view.render(info, template, headers=headers)
        assert headers.get('Content-Type') == 'text/html; charset=utf-8'
        headers = {}
        view.render(info, template, headers=headers, format='html')
        assert headers.get('Content-Type') == 'text/html; charset=utf-8'
        headers = {}
        view.render(info, template, headers=headers, format='html-strict')
        assert headers.get('Content-Type') == 'text/html; charset=utf-8'
        headers = {}
        view.render(info, template, headers=headers, format='xhtml')
        assert headers.get('Content-Type') == 'text/html; charset=utf-8'
        headers = {}
        view.render(info, template, headers=headers, format='xhtml-strict')
        assert headers.get('Content-Type') == 'text/html; charset=utf-8'
        headers = {}
        view.render(info, template, headers=headers, format='xml')
        assert headers.get('Content-Type') == 'text/xml; charset=utf-8'
        headers = {}
        view.render(info, template, headers=headers, format='json')
        assert headers.get('Content-Type') == 'application/json'
        config.update({'global':
            {'tg.format_mime_types': {'xhtml': 'application/xhtml+xml'}}})
        headers = {}
        view.render(info, template, headers=headers, format='xhtml')
        assert headers.get('Content-Type') == 'application/xhtml+xml; charset=utf-8'
        headers = {}
        view.render(info, template, headers=headers, format='xhtml-strict')
        assert headers.get('Content-Type') == 'application/xhtml+xml; charset=utf-8'
        config.update({'global': {'tg.format_mime_types': {}}})

    def test_plain_format(self):
        info = dict(someval="dumbos")
        template = "turbogears.tests.simple"
        headers = {}
        plain = view.render(info, template, headers=headers, format='plain')
        assert headers.get('Content-Type') == 'text/plain; charset=utf-8'
        assert plain.strip() == ('This is the groovy test template.'
            ' Paging all dumbos.')
        headers = {}
        text = view.render(info, template, headers=headers, format='text')
        assert headers.get('Content-Type') == 'text/plain; charset=utf-8'
        assert text == plain
        try:
            view.render(info, template, headers=headers, format='dumbo')
        except KeyError:
            pass
        else:
            assert False, "'dumbo' should not be accepted as format"



"""Localized formatting functions.

These functions extract localization data from config files located
in the data/directory.

"""

from turbogears.i18n.utils import get_locale
import pkg_resources
import os
import re


def is_locale_format(locale):
    """Check if locale is supported."""
    py_filename = pkg_resources.resource_filename(
        "turbogears.i18n.data", "%s.py" % locale)
    if os.path.exists(py_filename):
        return True
    pyc_filename = pkg_resources.resource_filename(
            "turbogears.i18n.data", "%s.pyc" % locale)
    if os.path.exists(pyc_filename):
        return True
    return False


def get_locale_module(locale):
    """Get i18n module supporting the locale."""
    try:
        # check if locale is supported. If not, check again with
        # first part of locale for example, "fi_FI" > "fi".
        if not is_locale_format(locale):
            locale = locale[:2]
        name = "turbogears.i18n.data.%s" % locale
        mod = __import__(name)
        parts = name.split(".")[1:]
        for p in parts:
            mod = getattr(mod, p)
        return mod
    except (ImportError, AttributeError):
        return None


def get(locale, name, default=None):
    """Get an attribute value for the locale."""
    locale = get_locale(locale)
    mod = get_locale_module(locale)
    return getattr(mod, name, default)


def get_countries(locale=None):
    """Get all supported countries.

    Returns a list of tuples, consisting of international country code
    and localized name, e.g. ('AU', 'Australia').

    """
    countries = get(locale, "countries", {}).items()
    countries.sort(lambda x, y: cmp(x[1], y[1]))
    return countries


def get_country(key, locale=None):
    """Get localized name of country based on international country code."""
    return get(locale, "countries", {})[key]


def get_languages(locale=None):
    """Get all supported languages.

    Returns a list of tuples, with language code and localized name,
    e.g. ('en', 'English').

    """
    languages = get(locale, "languages", {}).items()
    languages.sort(lambda x, y: cmp(x[1], y[1]))
    return languages


def get_language(key, locale=None):
    """Get localized name of language based on language code."""
    return get(locale, "languages", {})[key]


def get_month_names(locale=None):
    """Get list of full month names, starting with January."""
    return get(locale, "months", [])


def get_abbr_month_names(locale=None):
    """Get list of abbreviated month names, starting with Jan."""
    return get(locale, "abbrMonths", [])


def get_weekday_names(locale=None):
    """Get list of full weekday names."""
    return get(locale, "days", [])


def get_abbr_weekday_names(locale=None):
    """Get list of abbreviated weekday names."""
    return get(locale, "abbrDays", get_weekday_names(locale))


def get_decimal_format(locale=None):
    """Get decimal point for the locale."""
    return get(locale, "numericSymbols", {}).get("decimal", ".")


def get_group_format(locale=None):
    """Get digit group separator for thousands for the locale."""
    return get(locale, "numericSymbols", {}).get("group", ",")


def format_number(value, locale=None):
    """Get number formatted with grouping for thousands.

    E.g. 5000000 will be formatted as 5,000,000.

    """
    gf = get_group_format(locale)
    thou = re.compile(r"([0-9])([0-9][0-9][0-9]([%s]|$))" % gf).search
    v = str(value)
    mo = thou(v)
    while mo is not None:
        i = mo.start(0)
        v = v[:i+1] + gf + v[i+1:]
        mo = thou(v)
    return unicode(v)


def format_decimal(value, num_places, locale=None):
    """Get number formatted with grouping for thousands and decimal places.

    E.g. 5000000.898 will be formatted as 5,000,000.898.

    """
    v = ("%%.%df" % num_places) % value
    if num_places == 0:
        return format_number(v, locale=locale)
    num, decimals = v.split(".", 1)
    return format_number(num, locale) + unicode(
        get_decimal_format(locale) + decimals)


def format_currency(value, locale=None):
    """Get formatted currency value."""
    return format_decimal(value, 2, locale)


def parse_number(value, locale=None):
    """Take localized number string and return a long integer.

    Throws ValueError if bad format.

    """
    return long(value.replace(get_group_format(locale), ""))

def parse_decimal(value, locale=None):
    """Take localized decimal string and return a float.

    Throws ValueError if bad format.

    """
    value = value.replace(get_group_format(locale), "")
    value = value.replace(get_decimal_format(locale), ".")
    return float(value)


def get_date_format(format, locale=None):
    """Get localized date format."""
    formats = get(locale, "dateFormats", {})
    return formats.get(format, None)


def format_date(dt, format="medium", locale=None,
        time_format="", date_format=""):
    """Get formatted date value.

    format can be "full", "long", "medium" or "short".
    To have complete control over formatting,
    use time_format and date_format parameters.

    @param dt: datetime
    @type dt: datetime.datetime

    @param format: format("full", "long", "medium", "short")
    @type format: string

    @param locale: the locale
    @type locale: string

    @param time_format: standard time formatting string, e.g. %H:%M
    @type time_format:s tring

    @param time_format: date formatting template string.
    Template variables include standard date formatting string like %d or %Y
    plus a few locale-specific names:
    %%(abbrmonthname)s, %%(dayname)s, %%(abbrmonthname)s and %%(monthname)s.
    @type time_format: string

    """
    pattern = date_format or get_date_format(format, locale)
    if not pattern:
        return str(dt)
    month = dt.month - 1
    weekday = dt.weekday()
    # becasue strftime() accepts str only but not unicode,
    # we encode string to utf-8 and then decode back
    date_str = dt.strftime(pattern.encode('utf8') + time_format)
    return date_str.decode('utf8') % dict(
        monthname=get_month_names(locale)[month],
        abbrmonthname=get_abbr_month_names(locale)[month],
        dayname=get_weekday_names(locale)[weekday],
        abbrdayname=get_abbr_weekday_names(locale)[weekday])



import unittest
import formencode
import cherrypy
import pkg_resources
from turbogears import config, controllers, database, \
    error_handler, exception_handler, expose, flash, redirect, \
    startup, testutil, url, validate, validators


class SubApp(controllers.RootController):

    [expose()]
    def index(self):
        return url("/Foo/")


class MyRoot(controllers.RootController):

    [expose()]
    def index(self):
        pass

    def validation_error_handler(self, tg_source, tg_errors, *args, **kw):
        self.functionname = tg_source.__name__
        self.values = kw
        self.errors = tg_errors
        return "Error Message"

    [expose(template="turbogears.tests.simple", allow_json=True)]
    def test(self):
        return dict(title="Foobar", mybool=False, someval="niggles")

    [expose(html="turbogears.tests.simple")]
    def test_deprecated(self):
        return dict(title="Oldbar", mybool=False, someval="giggles")

    [expose()]
    def invalid(self):
        return None

    [expose()]
    def pos(self, posvalue):
        self.posvalue = posvalue
        return ""

    [expose()]
    def servefile(self, tg_exceptions=None):
        self.servedit = True
        self.serve_exceptions = tg_exceptions
        return cherrypy.lib.cptools.serveFile(
            pkg_resources.resource_filename(
                "turbogears.tests", "test_controllers.py"))

    [expose(content_type='text/plain')]
    def basestring(self):
        return 'hello world'

    [expose(content_type='text/plain')]
    def list(self):
        return ['hello', 'world']

    [expose(content_type='text/plain')]
    def generator(self):
        yield 'hello'
        yield 'world'

    [expose()]
    def unicode(self):
        cherrypy.response.headers["Content-Type"] = "text/html"
        return u'\u00bfHabla espa\u00f1ol?'

    [expose()]
    def returnedtemplate(self):
        return dict(title="Foobar", mybool=False, someval="foo",
            tg_template="turbogears.tests.simple")

    [expose()]
    def returnedtemplate_short(self):
        return dict(title="Foobar", mybool=False, someval="foo",
            tg_template="turbogears.tests.simple")

    [expose(template="turbogears.tests.simple")]
    def exposetemplate_short(self):
        return dict(title="Foobar", mybool=False, someval="foo")

    [expose()]
    [validate(validators={'value': validators.StringBoolean()})]
    def istrue(self, value):
        self.value = value
        return str(value)
    istrue = error_handler(validation_error_handler)(istrue)

    [expose()]
    [validate(validators={'value': validators.StringBoolean()})]
    def nestedcall(self, value):
        return self.istrue(str(value))

    [expose()]
    [validate(validators={'value': validators.StringBoolean()})]
    def errorchain(self, value):
        return "No Error"
    errorchain = error_handler(istrue)(errorchain)

    [expose(format="json", template="turbogears.tests.simple")]
    def returnjson(self):
        return dict(title="Foobar", mybool=False, someval="foo",
            tg_template="turbogears.tests.simple")

    [expose(template="turbogears.tests.simple", allow_json=False)]
    def allowjson(self):
        return dict(title="Foobar", mybool=False, someval="foo",
             tg_template="turbogears.tests.simple")

    [expose(format="json")]
    def impliedjson(self):
        return dict(title="Blah")

    [expose('json')]
    def explicitjson(self):
        return dict(title="Blub")

    [expose(template="turbogears.tests.simple", allow_json=True)]
    def jsonerror_handler(self):
        return dict(someval="errors")

    [expose(allow_json=True)]
    def jsonerror(self):
        raise ValueError
    jsonerror = exception_handler(jsonerror_handler)(jsonerror)

    [expose(content_type="xml/atom")]
    def contenttype(self):
        return "Foobar"

    [expose()]
    [validate(validators={
        "firstname": validators.String(min=2, not_empty=True),
        "lastname": validators.String()})]
    def save(self, submit, firstname, lastname="Miller"):
        self.submit = submit
        self.firstname = firstname
        self.lastname = lastname
        self.fullname = "%s %s" % (self.firstname, self.lastname)
        return self.fullname
    save = error_handler(validation_error_handler)(save)

    class Registration(formencode.Schema):
        allow_extra_fields = True
        firstname = validators.String(min=2, not_empty=True)
        lastname = validators.String()

    [expose()]
    [validate(validators=Registration())]
    def save2(self, submit, firstname, lastname="Miller"):
        return self.save(submit, firstname, lastname)
    save2 = error_handler(validation_error_handler)(save2)

    [expose(template="turbogears.tests.simple")]
    def useother(self):
        return dict(tg_template="turbogears.tests.othertemplate")

    [expose(template="cheetah:turbogears.tests.simplecheetah")]
    def usecheetah(self):
        return dict(someval="chimps")

    rwt_called = 0
    def rwt(self, func, *args, **kw):
        self.rwt_called += 1
        func(*args, **kw)

    [expose(template="turbogears.tests.simple", allow_json=True)]
    def flash_plain(self):
        flash("plain")
        return dict(title="Foobar", mybool=False, someval="niggles")

    [expose(template="turbogears.tests.simple", allow_json=True)]
    def flash_unicode(self):
        flash(u"\xfcnicode")
        return dict(title="Foobar", mybool=False, someval="niggles")

    [expose(template="turbogears.tests.simple", allow_json=True)]
    def flash_data_structure(self):
        flash(dict(uni=u"\xfcnicode", testing=[1, 2, 3]))
        return dict(title="Foobar", mybool=False, someval="niggles")

    [expose(template="turbogears.tests.simple", allow_json=True)]
    def flash_redirect(self):
        flash(u"redirect \xfcnicode")
        redirect("/flash_redirected?tg_format=json")

    [expose(template="turbogears.tests.simple", allow_json=True)]
    def flash_redirect_with_trouble_chars(self):
        flash(u"$foo, k\xe4se;\tbar!")
        redirect("/flash_redirected?tg_format=json")

    [expose(template="turbogears.tests.simple", allow_json=True)]
    def flash_redirected(self):
        return dict(title="Foobar", mybool=False, someval="niggles")

    def exc_h_value(self, tg_exceptions=None):
        """Exception handler for the ValueError in raise_value_exc"""
        return dict(handling_value=True, exception=str(tg_exceptions))

    [expose()]
    def raise_value_exc(self):
        raise ValueError('Some Error in the controller')
    raise_value_exc = exception_handler(exc_h_value,
        "isinstance(tg_exceptions, ValueError)")(raise_value_exc)

    def exc_h_key(self, tg_exceptions=None):
        """Exception handler for KeyErrors in  raise_all_exc"""
        return dict(handling_key=True, exception=str(tg_exceptions))

    def exc_h_index(self, tg_exceptions=None):
        """Exception handler for the ValueError in raise_value_exc"""
        return dict(handling_index=True, exception=str(tg_exceptions))

    [expose()]
    def raise_index_exc(self):
        raise IndexError('Some IndexError')
    raise_index_exc = exception_handler(exc_h_index,
        "isinstance(tg_exceptions, IndexError)")(raise_index_exc)

    [expose()]
    def raise_all_exc(self, num=2):
        num = int(num)
        if num < 2:
            raise ValueError('Inferior to 2')
        elif num == 2:
            raise IndexError('Equals to 2')
        elif num > 2:
            raise KeyError('No such number 2 in the integer range')
    raise_all_exc = exception_handler(exc_h_index,
        "isinstance(tg_exceptions, IndexError)")(raise_all_exc)
    raise_all_exc = exception_handler(exc_h_value,
        "isinstance(tg_exceptions, ValueError)")(raise_all_exc)
    raise_all_exc = exception_handler(exc_h_key,
        "isinstance(tg_exceptions, KeyError)")(raise_all_exc)

    [expose()]
    def internal_redirect(self, **kwargs):
        raise cherrypy.InternalRedirect('/internal_redirect_target')

    [expose()]
    def internal_redirect_target(self, **kwargs):
        return "redirected OK"

    [expose()]
    def redirect_to_path_str(self, path):
        raise redirect(path + '/index')

    [expose()]
    def redirect_to_path_list(self, path):
        raise redirect([path, 'index'])

    [expose()]
    def redirect_to_path_tuple(self, path):
        raise redirect((path, 'index'))


class TestRoot(unittest.TestCase):

    def setUp(self):
        cherrypy.root = None
        cherrypy.tree.mount_points = {}
        cherrypy.tree.mount(MyRoot(), "/")
        cherrypy.tree.mount(SubApp(), "/subthing")

    def tearDown(self):
        cherrypy.root = None
        cherrypy.tree.mount_points = {}

    def test_js_files(self):
        """Can access the JavaScript files"""
        testutil.create_request("/tg_js/MochiKit.js")
        assert cherrypy.response.headers["Content-Type"] in [
            "application/javascript", "application/x-javascript"]
        assert cherrypy.response.status == "200 OK"

    def test_json_output(self):
        testutil.create_request("/test?tg_format=json")
        import simplejson
        values = simplejson.loads(cherrypy.response.body[0])
        assert values == dict(title="Foobar", mybool=False,
            someval="niggles", tg_flash=None)
        assert cherrypy.response.headers["Content-Type"] == "application/json"

    def test_implied_json(self):
        testutil.create_request("/impliedjson?tg_format=json")
        assert '"title": "Blah"' in cherrypy.response.body[0]
        assert cherrypy.response.headers["Content-Type"] == "application/json"

    def test_explicit_json(self):
        testutil.create_request("/explicitjson")
        assert '"title": "Blub"' in cherrypy.response.body[0]
        assert cherrypy.response.headers["Content-Type"] == "application/json"
        testutil.create_request("/explicitjson?tg_format=json")
        assert '"title": "Blub"' in cherrypy.response.body[0]
        assert cherrypy.response.headers["Content-Type"] == "application/json"

    def test_allow_json(self):
        testutil.create_request("/allowjson?tg_format=json")
        assert cherrypy.response.headers["Content-Type"] == "text/html"

    def test_allow_json_config(self):
        """JSON output can be enabled via config."""
        config.update({'tg.allow_json':True})
        class JSONRoot(controllers.RootController):
            [expose(template="turbogears.tests.simple")]
            def allowjsonconfig(self):
                return dict(title="Foobar", mybool=False, someval="foo",
                     tg_template="turbogears.tests.simple")
        cherrypy.root = JSONRoot()
        testutil.create_request('/allowjsonconfig?tg_format=json')
        assert cherrypy.response.headers["Content-Type"] == "application/json"
        config.update({'tg.allow_json': False})

    def test_allow_json_config_false(self):
        """Make sure JSON can still be restricted with a global config on."""
        config.update({'tg.allow_json': True})
        class JSONRoot(controllers.RootController):
            [expose(template="turbogears.tests.simple")]
            def allowjsonconfig(self):
                return dict(title="Foobar", mybool=False, someval="foo",
                     tg_template="turbogears.tests.simple")
        cherrypy.root = JSONRoot()
        testutil.create_request('/allowjson?tg_format=json')
        assert cherrypy.response.headers["Content-Type"] == "text/html"
        config.update({'tg.allow_json': False})

    def test_json_error(self):
        """The error handler should return JSON if requested."""
        testutil.create_request("/jsonerror")
        assert cherrypy.response.headers["Content-Type"] == "text/html; charset=utf-8"
        assert "Paging all errors" in cherrypy.response.body[0]
        testutil.create_request("/jsonerror?tg_format=json")
        assert cherrypy.response.headers["Content-Type"] == "application/json"
        assert '"someval": "errors"' in cherrypy.response.body[0]

    def test_invalid_return(self):
        testutil.create_request("/invalid")
        assert cherrypy.response.status.startswith("500")

    def test_strict_parameters(self):
        config.update({"tg.strict_parameters": True})
        testutil.create_request(
            "/save?submit=save&firstname=Foo&lastname=Bar&badparam=1")
        assert cherrypy.response.status.startswith("500")
        assert not hasattr(cherrypy.root, "errors")

    def test_throw_out_random(self):
        """Can append random value to the URL to avoid caching problems."""
        testutil.create_request("/test?tg_random=1")
        assert "Paging all niggles" in cherrypy.response.body[0]
        config.update({"tg.strict_parameters": True})
        testutil.create_request("/test?tg_random=1")
        assert cherrypy.response.status.startswith("200")
        assert "Paging all niggles" in cherrypy.response.body[0]
        testutil.create_request("/test?tg_not_random=1")
        assert cherrypy.response.status.startswith("500")
        assert not hasattr(cherrypy.root, "errors")

    def test_ignore_parameters(self):
        config.update({"tg.strict_parameters": True})
        testutil.create_request("/test?ignore_me=1")
        assert cherrypy.response.status.startswith("500")
        assert not hasattr(cherrypy.root, "errors")
        config.update({"tg.ignore_parameters": ['ignore_me', 'me_too']})
        testutil.create_request("/test?ignore_me=1")
        assert "Paging all niggles" in cherrypy.response.body[0]
        testutil.create_request("/test?me_too=1")
        assert cherrypy.response.status.startswith("200")
        assert "Paging all niggles" in cherrypy.response.body[0]
        testutil.create_request("/test?me_not=1")
        assert cherrypy.response.status.startswith("500")
        assert not hasattr(cherrypy.root, "errors")

    def test_retrieve_dict_directly(self):
        d = testutil.call(cherrypy.root.returnjson)
        assert d["title"] == "Foobar"

    def test_template_output(self):
        testutil.create_request("/test")
        assert "Paging all niggles" in cherrypy.response.body[0]

    def test_template_deprecated(self):
        testutil.create_request("/test_deprecated")
        assert "Paging all giggles" in cherrypy.response.body[0]

    def test_unicode(self):
        testutil.create_request("/unicode")
        firstline = cherrypy.response.body[0].split('\n')[0].decode('utf-8')
        assert firstline == u'\u00bfHabla espa\u00f1ol?'

    def test_default_format(self):
        """The default format can be set via expose"""
        testutil.create_request("/returnjson")
        firstline = cherrypy.response.body[0]
        assert '"title": "Foobar"' in firstline
        testutil.create_request("/returnjson?tg_format=html")
        firstline = cherrypy.response.body[0]
        assert '"title": "Foobar"' not in firstline

    def test_content_type(self):
        """The content-type can be set via expose"""
        testutil.create_request("/contenttype")
        assert cherrypy.response.headers["Content-Type"] == "xml/atom"

    def test_returned_template_name(self):
        testutil.create_request("/returnedtemplate")
        data = cherrypy.response.body[0].lower()
        assert "<body>" in data
        assert 'groovy test template' in data

    def test_returned_template_short(self):
        testutil.create_request("/returnedtemplate_short")
        assert "Paging all foo" in cherrypy.response.body[0]

    def test_expose_template_short(self):
        testutil.create_request("/exposetemplate_short")
        assert "Paging all foo" in cherrypy.response.body[0]

    def test_validation(self):
        """Data can be converted and validated"""
        testutil.create_request("/istrue?value=true")
        assert cherrypy.root.value is True
        testutil.create_request("/istrue?value=false")
        assert cherrypy.root.value is False
        cherrypy.root = MyRoot()
        testutil.create_request("/istrue?value=foo")
        assert not hasattr(cherrypy.root, "value")
        assert cherrypy.root.functionname == "istrue"
        testutil.create_request("/save?submit=send&firstname=John&lastname=Doe")
        assert cherrypy.root.fullname == "John Doe"
        assert cherrypy.root.submit == "send"
        testutil.create_request("/save?submit=send&firstname=Arthur")
        assert cherrypy.root.fullname == "Arthur Miller"
        testutil.create_request("/save?submit=send&firstname=Arthur&lastname=")
        assert cherrypy.root.fullname == "Arthur "
        testutil.create_request("/save?submit=send&firstname=D&lastname=")
        assert len(cherrypy.root.errors) == 1
        assert cherrypy.root.errors.has_key("firstname")
        assert "characters" in cherrypy.root.errors["firstname"].msg.lower()
        testutil.create_request("/save?submit=send&firstname=&lastname=")
        assert len(cherrypy.root.errors) == 1
        assert cherrypy.root.errors.has_key("firstname")

    def test_validation_chained(self):
        """Validation is not repeated if it already happened"""
        cherrypy.root.value = None
        testutil.create_request("/errorchain?value=true")
        assert cherrypy.root.value is None
        testutil.create_request("/errorchain?value=notbool")
        assert cherrypy.root.value == 'notbool'

    def test_validation_nested(self):
        """Validation is not repeated in nested method call"""
        cherrypy.root.value = None
        testutil.create_request("/nestedcall?value=true")
        assert cherrypy.root.value == 'True'
        testutil.create_request("/nestedcall?value=false")
        assert cherrypy.root.value == 'False'

    def test_validation_with_schema(self):
        """Data can be converted/validated with formencode.Schema instance"""
        testutil.create_request("/save2?submit=send&firstname=Joe&lastname=Doe")
        assert cherrypy.root.fullname == "Joe Doe"
        assert cherrypy.root.submit == "send"
        testutil.create_request("/save2?submit=send&firstname=Arthur&lastname=")
        assert cherrypy.root.fullname == "Arthur "
        testutil.create_request("/save2?submit=send&firstname=&lastname=")
        assert len(cherrypy.root.errors) == 1
        assert cherrypy.root.errors.has_key("firstname")
        testutil.create_request("/save2?submit=send&firstname=D&lastname=")
        assert len(cherrypy.root.errors) == 1
        assert cherrypy.root.errors.has_key("firstname")

    def test_other_template(self):
        """'tg_template' in a returned dict will use the template specified there"""
        testutil.create_request("/useother")
        assert "This is the other template" in cherrypy.response.body[0]

    def test_cheetah_template(self):
        """Cheetah templates can be used as well"""
        testutil.create_request("/usecheetah")
        body = cherrypy.response.body[0]
        assert "This is the Cheetah test template." in body
        assert "Paging all chimps." in body

    def test_run_with_trans(self):
        """run_with_transaction is called only on topmost exposed method"""
        oldrwt = database.run_with_transaction
        database.run_with_transaction = cherrypy.root.rwt
        testutil.create_request("/nestedcall?value=true")
        database.run_with_transaction = oldrwt
        assert cherrypy.root.value
        assert cherrypy.root.rwt_called == 1

    def test_positional(self):
        """Positional parameters should work"""
        testutil.create_request("/pos/foo")
        assert cherrypy.root.posvalue == "foo"

    def test_flash_plain(self):
        """flash with strings should work"""
        testutil.create_request("/flash_plain?tg_format=json")
        import simplejson
        values = simplejson.loads(cherrypy.response.body[0])
        assert values["tg_flash"] == "plain"
        assert not cherrypy.response.simple_cookie.has_key("tg_flash")

    def test_flash_unicode(self):
        """flash with unicode objects should work"""
        testutil.create_request("/flash_unicode?tg_format=json")
        import simplejson
        values = simplejson.loads(cherrypy.response.body[0])
        assert values["tg_flash"] == u"\xfcnicode"
        assert not cherrypy.response.simple_cookie.has_key("tg_flash")

    def test_flash_on_redirect(self):
        """flash must survive a redirect"""
        testutil.create_request("/flash_redirect?tg_format=json")
        assert cherrypy.response.status.startswith("302")
        testutil.create_request(cherrypy.response.headers["Location"],
            headers=dict(Cookie=cherrypy.response.simple_cookie.output(
                header="").strip()))
        import simplejson
        values = simplejson.loads(cherrypy.response.body[0])
        assert values["tg_flash"] == u"redirect \xfcnicode"

    def test_flash_redirect_with_trouble_chars(self):
        """flash redirect with chars that can cause troubles in cookies"""
        testutil.create_request("/flash_redirect_with_trouble_chars?tg_format=json")
        assert cherrypy.response.status.startswith("302")
        value = cherrypy.response.simple_cookie["tg_flash"].value
        assert '$' not in value
        assert ',' not in value and ';' not in value
        assert ' ' not in value and '\t' not in value
        assert 'foo' in value and 'bar' in value
        assert u'k\xe4se'.encode('utf-8') in value
        assert '!' in value
        testutil.create_request(cherrypy.response.headers["Location"],
            headers=dict(Cookie=cherrypy.response.simple_cookie.output(
                header="").strip()))
        import simplejson
        values = simplejson.loads(cherrypy.response.body[0])
        assert values["tg_flash"] == u"$foo, k\xe4se;\tbar!"

    def test_double_flash(self):
        """latest set flash should have precedence"""
        # Here we are calling method that sets a flash message. However flash
        # cookie is still there. Turbogears should discard old flash message
        # from cookie and use new one, set by flash_plain().
        testutil.create_request("/flash_plain?tg_format=json",
            headers=dict(Cookie='tg_flash="old flash"; Path=/;'))
        import simplejson
        values = simplejson.loads(cherrypy.response.body[0])
        assert values["tg_flash"] == "plain"
        assert cherrypy.response.simple_cookie.has_key("tg_flash"), \
                "Cookie clearing request should be present"
        flashcookie = cherrypy.response.simple_cookie['tg_flash']
        assert flashcookie['expires'] == 0

    def test_set_kid_outputformat_in_config(self):
        """the outputformat for kid can be set in the config"""
        config.update({'kid.outputformat': 'xhtml'})
        testutil.create_request('/test')
        response = cherrypy.response.body[0]
        assert '<!DOCTYPE html PUBLIC "-//W3C//DTD XHTML ' in response
        config.update({'kid.outputformat': 'html'})
        testutil.create_request('/test')
        response = cherrypy.response.body[0]
        assert  '<!DOCTYPE HTML PUBLIC "-//W3C//DTD HTML ' in response
        assert '    This is the groovy test ' in response
        config.update({'kid.outputformat': 'html compact'})
        testutil.create_request('/test')
        response = cherrypy.response.body[0]
        assert  '<!DOCTYPE HTML PUBLIC "-//W3C//DTD HTML ' in response
        assert 'This is the groovy test ' in response
        assert '    ' not in response

    def test_fileserving(self):
        testutil.create_request("/servefile")
        assert cherrypy.root.servedit
        assert not cherrypy.root.serve_exceptions

    def test_basestring(self):
        testutil.create_request("/basestring")
        assert cherrypy.response.body[0] == 'hello world'
        assert cherrypy.response.headers["Content-Type"] == "text/plain"

    def test_list(self):
        testutil.create_request("/list")
        assert cherrypy.response.body[0] == 'helloworld'
        assert cherrypy.response.headers["Content-Type"] == "text/plain"

    def test_generator(self):
        testutil.create_request("/generator")
        assert cherrypy.response.body[0] == 'helloworld'
        assert cherrypy.response.headers["Content-Type"] == "text/plain"

    def test_internal_redirect(self):
        """regression test for #1022, #1407 and #1598"""
        testutil.create_request("/internal_redirect")
        firstline = cherrypy.response.body[0]
        assert "redirected OK" in firstline

    def test_internal_redirect_nested_variables(self):
        """regression test for #1022, #1407 and #1598"""
        testutil.create_request(
            "/internal_redirect?a=1&a-1.b=2&a-2.c=3&a-2.c-1=4")
        firstline = cherrypy.response.body[0]
        assert "redirected OK" in firstline

    def test_exc_value(self):
        """Exception is handled gracefully by the right exception handler."""
        testutil.create_request("/raise_value_exc")
        assert 'handling_value' in cherrypy.response.body[0]

    def test_exc_index(self):
        """Exception is handled gracefully by the right exception handler."""
        testutil.create_request("/raise_index_exc")
        assert 'handling_index' in cherrypy.response.body[0]

    def test_exc_all(self):
        """Test a controller that is protected by multiple exception handlers.

        It should raise either of the 3 exceptions but all should be handled
        by their respective handlers without problem...

        """
        testutil.create_request("/raise_all_exc?num=1")
        assert 'handling_value' in cherrypy.response.body[0]
        testutil.create_request("/raise_all_exc?num=2")
        assert 'handling_index' in cherrypy.response.body[0]
        testutil.create_request("/raise_all_exc?num=3")
        assert 'handling_key' in cherrypy.response.body[0]


class TestURLs(unittest.TestCase):

    def setUp(self):
        cherrypy.tree.mount_points = {}
        cherrypy.root = MyRoot()
        cherrypy.root.subthing = SubApp()
        cherrypy.root.subthing.subsubthing = SubApp()

    def tearDown(self):
        config.update({"server.webpath": ""})
        startup.startTurboGears()

    def test_basic_urls(self):
        testutil.create_request("/")
        assert "/foo" == url("/foo")
        assert "foo/bar" == url(["foo", "bar"])
        assert url("/foo", bar=1, baz=2) in \
            ["/foo?bar=1&baz=2", "/foo?baz=2&bar=1"]
        assert url("/foo", dict(bar=1, baz=2)) in \
            ["/foo?bar=1&baz=2", "/foo?baz=2&bar=1"]
        assert url("/foo", dict(bar=1, baz=None)) == "/foo?bar=1"

    def test_url_without_request_available(self):
        cherrypy.serving.request = None
        assert url("/foo") == "/foo"

    def test_approots(self):
        testutil.create_request("/subthing/")
        assert cherrypy.response.status.startswith("200")
        assert url("foo") == "foo"
        assert url("/foo") == "/subthing/foo"
        testutil.create_request("/nosubthing/")
        assert cherrypy.response.status.startswith("404")
        assert url("foo") == "foo"
        assert url("/foo") == "/foo"

    def test_lower_approots(self):
        testutil.create_request("/subthing/subsubthing/")
        assert url("/foo") == "/subthing/subsubthing/foo"

    def test_approots_with_path(self):
        config.update({"server.webpath": "/coolsite/root"})
        startup.startTurboGears()
        testutil.create_request("/coolsite/root/subthing/")
        assert url("/foo") == "/coolsite/root/subthing/foo"

    def test_redirect(self):
        config.update({"server.webpath": "/coolsite/root"})
        startup.startTurboGears()
        testutil.create_request("/coolsite/root/subthing/")
        try:
            redirect("/foo")
            assert False, "redirect exception should have been raised"
        except cherrypy.HTTPRedirect, e:
            assert "http://localhost/coolsite/root/subthing/foo" in e.urls
        try:
            raise redirect("/foo")
            assert False, "redirect exception should have been raised"
        except cherrypy.HTTPRedirect, e:
            assert "http://localhost/coolsite/root/subthing/foo" in e.urls
        try:
            redirect("foo")
            assert False, "redirect exception should have been raised"
        except cherrypy.HTTPRedirect, e:
            assert "http://localhost/coolsite/root/subthing/foo" in e.urls

    def test_redirect_to_path(self):
        for path_type in ('str', 'list', 'tuple'):
            for path in ('subthing', '/subthing'):
                url = "/redirect_to_path_%s?path=%s" % (path_type, path)
                testutil.create_request(url)
                assert cherrypy.response.status.startswith("302"), url
                assert (cherrypy.response.headers['Location']
                    == 'http://localhost/subthing/index'), url

    def test_multi_values(self):
        testutil.create_request("/")
        assert url("/foo", bar=[1, 2]) in \
            ["/foo?bar=1&bar=2", "/foo?bar=2&bar=1"]
        assert url("/foo", bar=("asdf", "qwer")) in \
            ["/foo?bar=qwer&bar=asdf", "/foo?bar=asdf&bar=qwer"]

    def test_unicode(self):
        """url() can handle unicode parameters"""
        testutil.create_request("/")
        assert url('/', x=u'\N{LATIN SMALL LETTER A WITH GRAVE}'
            u'\N{LATIN SMALL LETTER E WITH GRAVE}'
            u'\N{LATIN SMALL LETTER I WITH GRAVE}'
            u'\N{LATIN SMALL LETTER O WITH GRAVE}'
            u'\N{LATIN SMALL LETTER U WITH GRAVE}') \
            == '/?x=%C3%A0%C3%A8%C3%AC%C3%B2%C3%B9'

    def test_list(self):
        """url() can handle list parameters, with unicode too"""
        testutil.create_request("/")
        assert url('/', foo=['bar', u'\N{LATIN SMALL LETTER A WITH GRAVE}']
            ) == '/?foo=bar&foo=%C3%A0'

    def test_existing_query_string(self):
        """url() can handle URL with existing query string"""
        testutil.create_request("/")
        test_url = url('/foo', {'first': 1})
        assert url(test_url, {'second': 2}) == '/foo?first=1&second=2'

    def test_url_kwargs_overwrite_tgparams(self):
        """Keys in tgparams in call to url() overwrite kw args"""
        params = {'spamm': 'eggs'}
        assert 'spamm=ham' in url('/foo', params, spamm='ham')

    def test_url_doesnt_change_tgparams(self):
        """url() does not change the dict passed as second arg"""
        params = {'spamm': 'eggs'}
        assert 'foo' in url('/foo', params, spamm='ham')
        assert params['spamm'] == 'eggs'

    def test_index_trailing_slash(self):
        """If there is no trailing slash on an index method call, redirect"""
        cherrypy.root = SubApp()
        cherrypy.root.foo = SubApp()
        testutil.create_request("/foo")
        assert cherrypy.response.status.startswith("302")

    def test_can_use_internally_defined_arguments(self):
        """Can use argument names that are internally used by TG in controllers"""

        class App(controllers.RootController):

            [expose()]
            def index(self, **kw):
                return "\n".join(["%s:%s" % i for i in kw.iteritems()])

        cherrypy.root = App()
        testutil.create_request("/?format=foo&template=bar&fragment=boo")
        output = cherrypy.response.body[0]
        assert "format:foo" in output
        assert "template:bar" in output
        assert "fragment:boo" in output



import itertools

import cherrypy

from turbogears import controllers, expose, widgets, validators, view
from turbogears.testutil import call, catch_validation_errors

try:
    set
except NameError: # Python 2.3
    from sets import Set as set

class Request:
    input_values = {}
    validation_errors = {}

oldrequest = None

def setup_module():
    global oldrequest
    oldrequest = cherrypy.request
    cherrypy.request = Request()
    if not view.engines:
        view.load_engines()

def teardown_module():
    global oldrequest
    cherrypy.request = oldrequest

def test_rendering_without_engine():
    """Helpful error when rendering widgets with no templating engine loaded"""
    from turbogears import view
    engines = view.engines
    view.engines = {}
    try:
        widgets.CSSLink("foo")()
    except Exception, msg:
        msg = str(msg)
    else:
        msg = 'No error'
    view.engines = engines
    assert 'templating engine is not yet loaded' in msg

def test_label():
    """Tests simple labels"""
    label = widgets.Label("foo")
    rendered = label.render("The Foo", format='xhtml')
    assert """<label id="foo" class="label">The Foo</label>""" == rendered

def test_default_value():
    """Widgets can have a default value"""
    textfield = widgets.TextField("name")
    output = textfield.render(format='xhtml')
    assert 'value' not in output
    textfield = widgets.TextField("name", default="ed kowalczyk")
    output = textfield.render(format='xhtml')
    assert 'value="ed kowalczyk"' in output

def test_callable_default_value():
    """Widgets can have a callable default value"""
    textfield = widgets.TextField("name", default=lambda: "chad taylor")
    output = textfield.render(format='xhtml')
    assert 'value="chad taylor"' in output

def test_false_default_value():
    """Widgets can have a default value that evaluates to False"""
    textfield = widgets.TextField("name")
    assert textfield.default is None
    output = textfield.render(format='xhtml')
    assert 'value' not in output
    textfield = widgets.TextField("name", default=0)
    assert textfield.default == 0
    output = textfield.render(format='xhtml')
    assert 'value="0"' in output
    textfield = widgets.TextField("name", default="")
    assert textfield.default == ""
    output = textfield.render(format='xhtml')
    assert 'value=""' in output
    msfield = widgets.MultipleSelectField("name", default=[], options=(1,2,3))
    assert msfield.default == []
    output = msfield.render(format='xhtml')
    assert 'selected' not in output

def test_labeltext():
    """Label text defaults to the capitalized name"""
    textfield = widgets.TextField("name")
    assert textfield.label == "Name"

def test_validation():
    """Values can be converted to/from Python values"""
    textfield = widgets.TextField("age", validator=validators.Int())
    output = textfield.render(2, format="xhtml")
    assert 'value="2"' in output
    value = "2"
    value = textfield.validator.to_python(value)
    assert value == 2

def test_unicode_input():
    """Unicode values are rendered correctly"""
    tf = widgets.TextField("name", validator=validators.UnicodeString())
    output = tf.render(u'Pete \u011C', format='xhtml')
    assert 'value="Pete \xc4\x9c"' in output
    return # XXX: the folowing causes OTHER(!) tests to fail!
    try:
        print tf.render('Pete \xfe\xcd')
    except ValueError, e:
        pass
    else:
        assert False, "ValueError not raised: non-unicode input not detected"
    #tf2 = widgets.TextField("name", validator=validators.String())

# simon: failed inputs are no longer being removed.
#
#def test_failed_validation():
#    "If validation fails, the bad value should be removed from the input"
#    textfield = widgets.TextField("age", validator=validators.Int())
#    values = dict(age="ed")
#    try:
#        textfield.validate(values)
#    except validators.Invalid:
#        pass
#    assert not values.has_key("age")

def test_widget_css():
    """Widgets can require CSS resources"""
    css = widgets.CSSLink(mod=widgets.static, name="foo.css")
    css2 = widgets.CSSLink(mod=widgets.static, name="foo.css")
    assert css == css2
    cssset = set()
    cssset.add(css)
    cssset.add(css2)
    assert len(cssset) == 1
    css3 = widgets.CSSLink(mod=widgets.static, name="bar.css")
    assert css3 != css2
    css4 = widgets.CSSSource(src="foo.css")
    assert css != css4
    rendered = css.render(format='xhtml')
    assert 'link' in rendered
    assert 'href="/tg_widgets/turbogears.widgets/foo.css"' in rendered
    assert 'type="text/css"' in rendered
    assert 'rel="stylesheet"' in rendered
    assert 'media="all"' in rendered
    rendered = css.render(media="printer", format='xhtml')
    assert 'media="printer"' in rendered
    css = widgets.CSSSource("h1 { color: black }")
    rendered = css.render(format='xhtml')
    assert 'h1 { color: black }' in rendered

def test_widget_js():
    """Widgets can require JavaScript resources"""
    js = widgets.JSLink(mod=widgets.static, name="foo.js")
    js2 = widgets.JSLink(mod=widgets.static, name="foo.js")
    assert js2 == js2
    js2 = widgets.JSLink(mod=widgets.static, name="bar.js")
    assert js2 != js
    js2 = widgets.CSSLink(mod=widgets.static, name="foo.js")
    assert js2 != js
    js2 = widgets.JSSource(src="foo.js")
    assert js2 != js
    rendered = js.render(format='xhtml')
    expected = '<script src="/tg_widgets/turbogears.widgets/foo.js"' \
        ' type="text/javascript"></script>'
    assert rendered == expected
    js3 = widgets.JSLink(mod=widgets.static, name="foo.js",
        defer=False, charset=None)
    assert js3 == js
    rendered = js3.render(format='xhtml')
    assert rendered == expected
    js3 = widgets.JSLink(mod=widgets.static, name="foo.js", defer=True)
    assert js3 != js
    rendered = js3.render(format='html').lower()
    assert ' defer' in rendered \
        and expected == rendered.replace(' defer', '', 1)
    rendered = js3.render(format='xhtml')
    assert ' defer="defer"' in rendered \
        and expected == rendered.replace(' defer="defer"', '', 1)
    js3 = widgets.JSLink(mod=widgets.static, name="foo.js", charset='Latin-1')
    assert js3 != js
    rendered = js3.render(format='xhtml')
    assert ' charset="Latin-1"' in rendered \
        and expected == rendered.replace(' charset="Latin-1"', '', 1)
    js3 = widgets.JSSource("alert('hello');")
    assert js3 != js and js3 != js2
    rendered = js3.render(format='xhtml')
    expected = '<script type="text/javascript">alert(\'hello\');</script>'
    assert rendered == expected
    js4 = widgets.JSSource("alert('hello');", defer=False)
    assert js4 == js3
    rendered = js4.render(format='xhtml')
    assert rendered == expected
    js4 = widgets.JSSource("alert('hello');", defer=True)
    assert js4 != js3
    rendered = js4.render(format='html').lower()
    assert ' defer' in rendered \
        and expected == rendered.replace(' defer', '', 1)
    rendered = js4.render(format='xhtml')
    assert ' defer="defer"' in rendered \
        and expected == rendered.replace(' defer="defer"', '', 1)

def test_widgetslist_init():
    """Widget lists can be declared in various ways."""
    w = widgets.Widget(name="foo")
    widgetlist = widgets.WidgetsList(w)
    assert len(widgetlist) == 1
    assert widgetlist[0] == w
    widgetlist = widgets.WidgetsList([w])
    assert len(widgetlist) == 1
    assert widgetlist[0] == w
    w2 = widgets.Widget(name="bar")
    widgetlist = widgets.WidgetsList(w, w2)
    assert len(widgetlist) == 2
    assert widgetlist[0] == w
    assert widgetlist[1] == w2
    widgetlist = widgets.WidgetsList([w, w2])
    assert len(widgetlist) == 2
    assert widgetlist[0] == w
    assert widgetlist[1] == w2
    class W(widgets.WidgetsList):
        foo = w
        bar = w2
    widgetlist = W()
    assert len(widgetlist) == 2
    assert widgetlist[0] == w
    assert widgetlist[1] == w2

def test_widget_url():
    """It might be needed to insert an URL somewhere"""
    url = widgets.URLLink(link='http://www.turbogears.org')
    rendered = url.render(format='xhtml')
    expected = """<a href="http://www.turbogears.org"></a>"""
    assert rendered == expected
    url = widgets.URLLink(link='http://www.turbogears.org', text='TurboGears Website')
    rendered = url.render(format='xhtml')
    expected = """<a href="http://www.turbogears.org">TurboGears Website</a>"""
    assert rendered == expected
    url = widgets.URLLink(link='http://www.turbogears.org', text='TurboGears Website', target="_blank")
    rendered = url.render(format='xhtml')
    expected = """<a href="http://www.turbogears.org" target="_blank">TurboGears Website</a>"""
    assert rendered == expected

def test_submit():
    sb = widgets.SubmitButton()
    r = sb.render(format='xhtml')
    assert 'name' not in r
    assert 'id' not in r
    r = sb.render('Krakatoa', format='xhtml')
    assert 'id' not in r
    assert 'name' not in r
    sb = widgets.SubmitButton(name='blink')
    r = sb.render(format='xhtml')
    assert 'name="blink"' in r
    assert 'id="blink"' in r
    r = sb.render('Krakatoa', format='xhtml')
    assert 'name="blink"' in r
    assert 'id="blink"' in r
    sb = widgets.SubmitButton(name='submit')
    r = sb.render(format='xhtml')
    assert 'name="submit"' in r
    assert 'id="submit"' in r
    r = sb.render('Krakatoa', format='xhtml')
    assert 'name="submit"' in r
    assert 'id="submit"' in r
    sb = widgets.SubmitButton(default='Save')
    r = sb.render(format='xhtml')
    assert 'value="Save"' in r
    r = sb.render(value='Discard', format='xhtml')
    assert 'value="Discard"' in r

def test_threadsafety():
    """Widget attributes can't be changed after init, for threadsafety"""
    w = widgets.TextField("bar")
    w.display()
    try:
        w.name = "foo"
        assert False, "should have gotten an exception"
    except ValueError:
        pass

def test_checkbox():
    """A CheckBox has not a value and is not checked by default"""
    w = widgets.CheckBox("foo")
    output = w.render(format='xhtml')
    assert 'name="foo"' in output
    assert 'value' not in output
    assert 'checked' not in output
    output = w.render(value=True, format='xhtml')
    assert 'checked' in output
    w = widgets.CheckBox("foo", default=True)
    output = w.render(format='xhtml')
    assert 'checked' in output
    output = w.render(value=False, format='xhtml')
    assert 'checked' not in output
    #CheckBox should accept alternate validators
    value = w.validator.to_python('checked')
    assert value == True
    w = widgets.CheckBox("foo", validator=validators.NotEmpty())
    value = w.validator.to_python('checked')
    assert value == 'checked'

def test_field_class():
    """The class of a field corresponds to the name of its Python class"""
    w = widgets.TextField("bar")
    output = w.render(format='xhtml')
    assert 'class="%s"' % w.__class__.__name__.lower() in output

def test_field_id():
    """The id of a field corresponds to the name of the field"""
    w = widgets.TextField("bar")
    output = w.render(format='xhtml')
    assert 'id="bar"' in output

def test_override_name():
    """The name of a widget can be overridden when displayed"""
    w = widgets.DataGrid(name="bar", fields=[])
    output = w.render([], name='foo', format='xhtml')
    assert 'id="foo"' in output

def test_selection_field():
    """A selection field presents a list of options that can be changed
    dynamically. One or more options can be selected/checked by default
    or dynamically."""
    options = [(1, "python"), (2, "java"), (3, "pascal")]
    w = widgets.SingleSelectField(options=options)
    output = w.render(format='xhtml')
    assert 'python' in output
    assert 'java' in output
    assert 'pascal' in output
    output = w.render(value=2, format='xhtml')
    assert '<option value="1">' in output
    assert '<option selected="selected" value="2">' in output
    assert '<option value="3">' in output
    w = widgets.SingleSelectField(options=options, default=3)
    output = w.render(format='xhtml')
    assert '<option value="1">' in output
    assert '<option value="2">' in output
    assert '<option selected="selected" value="3">' in output
    output = w.render(options=options + [(4, "cobol"), (5, "ruby")],
        format='xhtml')
    assert 'python' in output
    assert 'java' in output
    assert 'pascal' in output
    assert 'cobol' in output
    assert 'ruby' in output
    output = w.render(options=options
        + [(4, "cobol"), (5, "ruby")], value=5, format='xhtml')
    assert '<option value="1">' in output
    assert '<option value="2">' in output
    assert '<option value="3">' in output
    assert '<option value="4">' in output
    assert '<option selected="selected" value="5">' in output
    w = widgets.MultipleSelectField(options=options, default=[1, 3])
    output = w.render(format='xhtml')
    assert '<option selected="selected" value="1">' in output
    assert '<option value="2">' in output
    assert '<option selected="selected" value="3">' in output
    output = w.render(options=options
        + [(4, "cobol"), (5, "ruby")], value=[2, 4, 5], format='xhtml')
    assert '<option value="1">' in output
    assert '<option selected="selected" value="2">' in output
    assert '<option value="3">' in output
    assert '<option selected="selected" value="4">' in output
    assert '<option selected="selected" value="5">' in output

def test_callable_options():
    """Widgets support callable options passed to the
    constructor or dynamically"""
    def options_func1():
        return [(1, "coke"), (2, "pepsi"), (3, "redbull")]
    def options_func2():
        return [(1, "python"), (2, "java"), (3, "pascal")]
    w = widgets.SingleSelectField(options=options_func1)
    output = w.render(format='xhtml')
    assert 'coke' in output
    assert 'pepsi' in output
    assert 'redbull' in output
    output = w.render(options=options_func2, format='xhtml')
    assert 'coke' not in output
    assert 'pepsi' not in output
    assert 'redbull' not in output
    assert 'python' in output
    assert 'java' in output
    assert 'pascal' in output

class TestParams:
    class A(widgets.Widget):
        template = """<tag xmlns:py="http://purl.org/kid/ns#" a="${a}" />"""
        params = ["a", "b", "c"]
    class B(widgets.Widget):
        params = ["b", "c", "d"]
    class C(A, B):
        params = ["c", "d", "e"]

    def test_building_params(self):
        """
        Tests that the list of params is built correctly. Must be the
        union of all params from all bases.
        """
        # for easy equivalenece testing
        self.C.params.sort()
        assert ''.join(self.C.params) == "abcde"

    def test_default_values(self):
        """
        Test that params which are not initialized at the ctor. nor
        at the subclass declarartion default to None.
        """
        a = self.A()
        assert a.a is None
        output = a.render(format='xhtml')
        assert 'a=' not in output

    def test_overridal(self):
        """
        Test we can override a template_var in the ctor, and at display.
        Both from the class and it's bases.
        """
        a = self.A(a="test")
        assert a.a == "test"
        c = self.C(a="test")
        assert c.a == "test"
        output = c.render(format='xhtml')
        assert 'a="test"' in output
        output = c.render(a="another", format='xhtml')
        assert 'a="another"' in output

def test_template_overridal():
    """Tests that we can override an instances template at construction time
    and get it automatically compiled."""
    new_template = """
    <label xmlns:py="http://purl.org/kid/ns#"
        for="${name}"
        class="${field_class}"
        py:content="value"
        custom_template="True"
    />
    """
    l = widgets.Label(template=new_template)
    output = l.render(format='xhtml')
    assert 'custom_template="True"' in output

def test_simple_widget_attrs():
    """A simple widget supports attributes passed to the constructor or at
    display time."""
    w = widgets.TextField(name="foo", attrs={'onchange':'python', 'size':'10'})
    output = w.render(format='xhtml')
    assert 'onchange="python"' in output
    assert 'size="10"' in output
    output = w.render(attrs={'onclick':'java'}, format='xhtml')
    assert 'onchange="python"' not in output
    assert 'size="10"' not in output
    assert 'onclick="java"' in output
    output = w.render(attrs={'onchange':'java', 'size':'50', 'alt':None},
        format='xhtml')
    assert 'onchange="java"' in output
    assert 'size="50"' in output
    assert 'alt' not in output
    assert 'onclick' not in output

def test_textfield():
    """Test form rendering inside a request."""
    #form name is prepended to element ids only when running inside a request.
    class MyField(widgets.WidgetsList):
        blonk = widgets.TextField()
    tf = widgets.ListForm(fields=MyField())

    class MyFieldOverrideName(widgets.WidgetsList):
        blonk = widgets.TextField(name="blink")
    tf2 = widgets.ListForm(fields=MyFieldOverrideName())

    class MyRoot(controllers.RootController):
        [expose()]
        def fields(self):
            return tf.render(format='xhtml')

        [expose()]
        def override(self):
            return tf2.render(format='xhtml')

    root = MyRoot()
    d = call(root.fields)
    assert 'name="blonk"' in d
    assert 'id="form_blonk"' in d

    d = call(root.override)
    assert 'name="blink"' in d
    assert 'id="form_blink"' in d

def test_textarea():
    w = widgets.TextArea(rows=20, cols=30)
    output = w.render(format='xhtml')
    assert 'rows="20"' in output
    assert 'cols="30"' in output
    output = w.render(rows=50, cols=50, format='xhtml')
    assert 'rows="50"' in output
    assert 'cols="50"' in output
    assert '> +++ </textarea>' in w.render(' +++ ', format='xhtml')

def test_render_field_for():
    """
    Using the render_field_for method of a FormFieldsContainer we can
    render the widget instance associated to a particular field name.
    """
    class MyFields(widgets.WidgetsList):
        name = widgets.TextField()
        age = widgets.TextArea()
    tf = widgets.ListForm(fields=MyFields())
    output = tf.render_field_for("name", format='xhtml')
    assert 'name="name"' in output
    assert '<input' in output
    assert 'type="text"' in output
    output = tf.render_field_for("name", attrs={'onclick':'hello'},
        format='xhtml')
    assert 'onclick="hello"' in output
    output = tf.render_field_for("age", format='xhtml')
    assert 'name="age"' in output
    assert '<textarea' in output
    output = tf.render_field_for("age", rows="1000", cols="2000",
        format='xhtml')
    assert 'rows="1000"' in output
    assert 'cols="2000"' in output

def test_css_classes():
    """A FormField supports css_classes, they are added after the original
    class. They can be provided at construction or at display time, the latter
    overrides the former but attrs overrides everything."""
    w = widgets.TextField(name="foo")
    output = w.render(format='xhtml')
    assert 'class="textfield"' in output
    w = widgets.TextField(name="foo", css_classes=["bar", "bye"])
    output = w.render(format='xhtml')
    assert 'class="textfield bar bye"' in output
    output = w.render(css_classes=["coke", "pepsi"], format='xhtml')
    assert 'class="textfield coke pepsi"' in output
    w = widgets.TextField(name="foo",
        css_classes=["bar", "bye"], attrs={'class':'again'})
    output = w.render(format='xhtml')
    assert 'class="again"' in output
    output = w.render(css_classes=["coke", "pepsi"], format='xhtml')
    assert 'class="again"' in output
    output = w.render(css_classes=["coke", "pepsi"],
        attrs={'class':'funny'}, format='xhtml')
    assert 'class="funny"' in output

def test_ticket272():
    """TextFields with a "name" attribute = "title" should be OK."""
    w = widgets.TableForm(fields=[widgets.TextField(name='title')])
    output = w.render(format='xhtml')
    assert 'value' not in output


class TestSchemaValidation:
    """Tests the validation of a CompoundWidget is done correctly with a
    Schema validator and no validators on the child widgets."""
    class Fields(widgets.WidgetsList):
        name = widgets.TextField()
        age = widgets.TextField()
        passwd = widgets.PasswordField()
        passwd2 = widgets.PasswordField()

    class FieldsSchema(validators.Schema):
        chained_validators = [validators.FieldsMatch('passwd', 'passwd2')]

        name = validators.UnicodeString()
        age = validators.Int()
        passwd = validators.NotEmpty()
        passwd2 = validators.UnicodeString()

    form = widgets.TableForm(fields=Fields(), validator=FieldsSchema())

    def test_goodvalues(self):
        values = dict(name=u'Jos\xc3\xa9', age="99", passwd="fado",
                      passwd2="fado")

        values, errors = catch_validation_errors(self.form, values)
        assert values['age'] == 99
        assert not errors

    def test_badvalues(self):
        values = dict(name=u'Jos\xc3\xa9', age="99", passwd="fado",
                      passwd2="fadO")

        values, errors = catch_validation_errors(self.form, values)
        assert "passwd2" in errors.keys()

class TestSchemaValidationWithChildWidgetsValidators:
    """Tests the validation of a CompoundWidget is done correctly with a
    Schema validator and independent validators on the each of the child
    widgets."""
    class Fields(widgets.WidgetsList):
        name = widgets.TextField(validator = validators.UnicodeString())
        age = widgets.TextField(validator = validators.Int())
        passwd = widgets.PasswordField(validator = validators.NotEmpty())
        passwd2 = widgets.PasswordField(validator = validators.UnicodeString())

    class FieldsSchema(validators.Schema):
        chained_validators = [validators.FieldsMatch('passwd', 'passwd2')]

    form = widgets.TableForm(fields=Fields(), validator=FieldsSchema())

    def test_goodvalues(self):
        values = dict(name=u'Jos\xc3\xa9', age="99", passwd="fado",
                      passwd2="fado")

        values, errors = catch_validation_errors(self.form, values)
        assert values['age'] == 99
        assert not errors.keys()

    def test_widget_validator_failure(self):
        values = dict(name=u'Jos\xc3\xa9', age="ninetynine", passwd="fado",
                      passwd2="fado")
        values, errors = catch_validation_errors(self.form, values)
        assert "age" in errors.keys()

    def test_widget_validator_and_schema_failure(self):
        values = dict(name=u'Jos\xc3\xa9', age="ninetynine", passwd="fado",
                      passwd2="fadO")
        values, errors = catch_validation_errors(self.form, values)
        assert "age" in errors.keys()
        assert "passwd2" in errors.keys()

def test_param_descriptor():
    class Base(widgets.Widget):
        params = ["param1", "param2"]
        param1 = "original"
        param2 = "original"

    class Sub(Base):
        param1 = lambda self: "overrided"
        param2 = "overrided"

    base = Base()
    assert base.param1 == "original", "descriptor is not created correctly"
    assert base.param2 == "original", "descriptor is not created correctly"

    sub = Sub()
    assert sub.param1 == "overrided", "callable params are not being overrided"
    assert sub.param2 == "overrided", "normal params are not being overrided"

def test_param_descriptor_mutable_class_attrs():
    class Wid(widgets.Widget):
        params = ['attrs']
        attrs = {}

    w1 = Wid(attrs={'test':True})
    w2 = Wid()

    assert w1.attrs == {'test':True}
    assert w2.attrs == {}

def test_param_descriptor_properties():
    class Wid(widgets.Widget):
        params = ['attrs']
        def _set_attrs(self, attrs):
            self._attrs = attrs
        def _get_attrs(self):
            return self._attrs
        attrs = property(_get_attrs, _set_attrs)

    w1 = Wid(attrs={'test':True})
    assert w1.attrs == {'test':True}

def test_dict_as_validator():
    class Foo(widgets.InputWidget):
        validator = validators.Int()
    a = Foo()
    b = Foo(validator=dict(not_empty=True))
    assert not a.validator.not_empty
    assert b.validator.not_empty
    assert a.validator is not b.validator

def test_params_doc():
    """Tests params_doc are picked from all bases giving priority to the
    widget's own."""
    class BaseA(widgets.Widget):
        params_doc = {'a':1}
    class BaseB(widgets.Widget):
        params_doc = {'b':2}
    class WidgetC(BaseA, BaseB):
        params_doc = {'c':3, 'a':4}
    widC = WidgetC()
    assert widC.params_doc == {'a':4, 'b':2, 'c':3}

def test_selectfield_with_with_non_iterable_option_elements():
    options = ["python", "java", "pascal"]
    w = widgets.SingleSelectField(options=options)
    output = w.render(format='xhtml')
    assert '<option value="python">' in output
    assert '<option value="java">' in output
    assert '<option value="pascal">' in output
    output = w.render(value="python", format='xhtml')
    assert '<option selected="selected" value="python">' in output
    assert '<option value="java">' in output
    assert '<option value="pascal">' in output
